# -*- coding: utf-8 -*-

from functools import update_wrapper

from django.conf import settings
from django.contrib import messages
from django.contrib.admin import ModelAdmin
from django.contrib.admin.options import InlineModelAdmin
from django.contrib.admin.sites import AdminSite
from django.contrib.admin.utils import unquote
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.http import Http404, HttpResponseRedirect
from django.template.defaultfilters import slugify
from django.template.response import TemplateResponse
from django.utils import six
from django.utils.encoding import force_text
from django.utils.html import escape
from django.utils.translation import ugettext as _
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_protect

from .page import Glitter
from .utils import JSONEncoderForHTML


class BlockAdminSite(AdminSite):
    change_form_template = 'blockadmin/change_form.html'

    def __init__(self, *args, **kwargs):
        # All blocks can be registered to this admin site
        self.block_list = {}

        super(BlockAdminSite, self).__init__(*args, **kwargs)

    # Use the block admin class by default
    def register(self, model_or_iterable, admin_class=None, **options):
        if not admin_class:
            admin_class = BlockModelAdmin

        super(BlockAdminSite, self).register(model_or_iterable, admin_class, **options)

    # Blocks from the site or other apps can be registered
    def register_block(self, block, category):
        if category not in self.block_list:
            self.block_list[category] = []

        self.block_list[category].append(block)

    def unregister_block(self, block, category):
        try:
            self.block_list[category].remove(block)
        except ValueError:
            pass

    # Remove all of the default admin URLs, we only want the model views for
    # the block admin.
    def get_urls(self):
        from django.conf.urls import url, include

        if settings.DEBUG:
            self.check_dependencies()

        def wrap(view, cacheable=False):
            def wrapper(*args, **kwargs):
                return self.admin_view(view, cacheable)(*args, **kwargs)
            return update_wrapper(wrapper, view)

        # Admin-site-wide views.
        urlpatterns = [
            url(r'^jsi18n/$', wrap(self.i18n_javascript, cacheable=True), name='jsi18n'),
        ]

        # Add in each model's views.
        for model, model_admin in six.iteritems(self._registry):
            urlpatterns += [
                url(r'^%s/%s/' % (model._meta.app_label, model._meta.model_name),
                    include(model_admin.urls))
            ]
        return urlpatterns

    # We have no logout here, so just raise PermissionDenied if needed
    def admin_view(self, view, cacheable=False):
        def inner(request, *args, **kwargs):
            if not self.has_permission(request):
                raise PermissionDenied
            return view(request, *args, **kwargs)
        if not cacheable:
            inner = never_cache(inner)
        # We add csrf_protect here so this function can be used as a utility
        # function for any view, without having to repeat 'csrf_protect'.
        inner = csrf_protect(inner)
        return update_wrapper(inner, view)


class BlockModelAdmin(ModelAdmin):
    # Keep block admin change forms in the blockadmin template directory
    @property
    def change_form_template(self):
        opts = self.model._meta
        app_label = opts.app_label

        return (
            "blockadmin/%s/%s/change_form.html" % (app_label, opts.object_name.lower()),
            "blockadmin/%s/change_form.html" % app_label,
            "blockadmin/change_form.html"
        )

    # We only want the change form for the block admin
    def get_urls(self):
        from django.conf.urls import url

        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)
            return update_wrapper(wrapper, view)

        info = self.model._meta.app_label, self.model._meta.model_name

        urlpatterns = [
            url(r'^(.+)/continue/$',
                wrap(self.continue_view),
                name='%s_%s_continue' % info),
            url(r'^(.+)/$',
                wrap(self.change_view),
                name='%s_%s_change' % info),
        ]
        return urlpatterns

    def log_change(self, request, object, message):
        pass

    def response_rerender(self, request, obj, template, extra_context=None):
        content_block = obj.content_block
        version = content_block.obj_version

        glitter = Glitter(version, request=request)
        columns = glitter.render(edit_mode=True, rerender=True)

        context = {
            'column': slugify(content_block.column),
            'rendered_json': JSONEncoderForHTML().encode({
                'content': columns[content_block.column],
            }),
        }

        if extra_context is not None:
            context.update(extra_context)

        return TemplateResponse(request, template, context, current_app=self.admin_site.name)

    # A redirect back to the edit view
    def continue_view(self, request, object_id):
        obj = self.get_object(request, unquote(object_id))

        if obj is None:
            raise Http404(_('%(name)s object with primary key %(key)r does not exist.') % {
                'name': force_text(self.opts.verbose_name),
                'key': escape(object_id)},
            )

        if not self.has_change_permission(request, obj):
            raise PermissionDenied

        opts = self.opts.app_label, self.opts.model_name
        change_url = reverse(
            'admin:%s_%s_change' % opts,
            args=(object_id,),
            current_app=self.admin_site.name)

        # This template will rerender the
        return self.response_rerender(request, obj, 'blockadmin/continue.html', {
            'change_url': change_url,
        })

    def has_change_permission(self, request, obj=None):
        # This shouldn't happen - but given that we need to find out the permissions for the glitter
        # object and not just this content block, just fail early incase something goes wrong.
        if obj is None:
            return False

        # Find the glitter object to see if they've got permission to edit that
        version = obj.content_block.obj_version
        glitter_obj = version.content_object
        opts = glitter_obj._meta.app_label, glitter_obj._meta.model_name
        return request.user.has_perm('%s.edit_%s' % opts)

    def change_view(self, request, object_id, form_url='', extra_context=None):
        """The 'change' admin view for this model."""
        obj = self.get_object(request, unquote(object_id))

        if obj is None:
            raise Http404(_('%(name)s object with primary key %(key)r does not exist.') % {
                'name': force_text(self.opts.verbose_name),
                'key': escape(object_id),
            })

        if not self.has_change_permission(request, obj):
            raise PermissionDenied

        content_block = obj.content_block
        version = content_block.obj_version

        # Version must not be saved, and must belong to this user
        if version.version_number or version.owner != request.user:
            raise PermissionDenied

        return super(BlockModelAdmin, self).change_view(request, object_id, form_url, extra_context)

    def response_change(self, request, obj):
        """Determine the HttpResponse for the change_view stage."""
        opts = self.opts.app_label, self.opts.model_name
        pk_value = obj._get_pk_val()

        if "_continue" in request.POST:
            msg = _('The %(name)s block was changed successfully. You may edit it again below.') % {
                'name': force_text(self.opts.verbose_name),
            }
            self.message_user(request, msg, messages.SUCCESS)

            # We redirect to the save and continue page, which updates the
            # parent window in javascript and redirects back to the edit page
            # in javascript.
            return HttpResponseRedirect(reverse(
                'admin:%s_%s_continue' % opts,
                args=(pk_value,),
                current_app=self.admin_site.name))

        # Update column and close popup - don't bother with a message as they won't see it
        return self.response_rerender(request, obj, 'admin/glitter/update_column.html')


class InlineBlockModelAdmin(InlineModelAdmin):
    # For inline objects we always allow adding/editing/deleting. This is an additional check done
    # after the permissions for the object has been checked - everyone needs to be able to edit the
    # same content on the same page, so just allow all inlines.

    def has_add_permission(self, request):
        return True

    def has_change_permission(self, request, obj=None):
        return True

    def has_delete_permission(self, request, obj=None):
        return True


class StackedInline(InlineBlockModelAdmin):
    template = 'admin/edit_inline/stacked.html'


class TabularInline(InlineBlockModelAdmin):
    template = 'admin/edit_inline/tabular.html'


site = BlockAdminSite(name='block_admin')



# -*- coding: utf-8 -*-

from glitter import block_admin

from glitter.assets.widgets import ImageRelatedFieldWidgetWrapper, ImageSelect
from .forms import ImageBlockForm
from .models import ImageBlock


class ImageBlockAdmin(block_admin.BlockModelAdmin):
    form = ImageBlockForm

    def formfield_for_dbfield(self, db_field, **kwargs):
        """
        Hook for specifying the form Field instance for a given database Field
        instance.

        If kwargs are given, they're passed to the form Field's constructor.
        """
        formfield = super(ImageBlockAdmin, self).formfield_for_dbfield(db_field, **kwargs)
        if db_field.name == 'image':
            formfield.widget = ImageRelatedFieldWidgetWrapper(
                ImageSelect(), db_field.rel, self.admin_site, can_add_related=True,
                can_change_related=True,
            )
        return formfield


block_admin.site.register(ImageBlock, ImageBlockAdmin)
block_admin.site.register_block(ImageBlock, 'Common')



# -*- coding: utf-8 -*-

from django import forms

from .models import ImageBlock


class ImageBlockForm(forms.ModelForm):
    class Meta:
        model = ImageBlock
        exclude = ()

    def __init__(self, *args, **kwargs):
        self.base_fields['image'].widget.attrs['class'] = 'image-related-field'
        super(ImageBlockForm, self).__init__(*args, **kwargs)



# -*- coding: utf-8 -*-

from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from mptt.managers import TreeManager
from mptt.models import MPTTModel, TreeForeignKey

from glitter.mixins import GlitterMixin
from glitter.models import Version
from .validators import validate_page_url


class PageManager(TreeManager):
    def published(self):
        return self.filter(published=True).exclude(current_version=None)

    def unpublished(self):
        return self.filter(published=True, current_version__isnull=True)


@python_2_unicode_compatible
class Page(MPTTModel, GlitterMixin):
    url = models.CharField('URL', max_length=100, unique=True, validators=[validate_page_url])
    title = models.CharField(max_length=100)
    parent = TreeForeignKey('self', null=True, blank=True, related_name='children')
    login_required = models.BooleanField(default=False)
    show_in_navigation = models.BooleanField(default=True, db_index=True)
    unpublished_count = models.PositiveIntegerField(default=0, editable=False)

    objects = PageManager()

    def __str__(self):
        return '%s -- %s' % (self.title, self.url)

    class Meta(GlitterMixin.Meta):
        verbose_name = 'page'
        ordering = ('url',)
        permissions = (
            ('view_protected_page', 'Can view protected page'),
        )

    def get_absolute_url(self):
        return self.url

    def save(self, *args, **kwargs):
        # Find the number of unpublished pages
        unpublished_pages = Version.objects.filter(
            content_type=ContentType.objects.get_for_model(self), object_id=self.id).exclude(
            version_number__isnull=True)

        if self.current_version:
            unpublished_pages = unpublished_pages.filter(
                version_number__gt=self.current_version.version_number)

        self.unpublished_count = unpublished_pages.count()

        super(Page, self).save(*args, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import glitter.blocks.video.validators


class Migration(migrations.Migration):

    dependencies = [
        ('glitter', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Video',
            fields=[
                ('id', models.AutoField(auto_created=True, serialize=False, primary_key=True, verbose_name='ID')),
                ('url', models.URLField(validators=[glitter.blocks.video.validators.validate_url], help_text='YouTube, Vimeo videos only', verbose_name='URL')),
                ('html', models.TextField(editable=False)),
                ('content_block', models.ForeignKey(editable=False, to='glitter.ContentBlock', null=True)),
            ],
            options={
                'verbose_name': 'video',
            },
        ),
    ]



# -*- coding: utf-8 -*-

from django.apps import AppConfig


class PagesConfig(AppConfig):
    name = 'glitter.pages'
    label = 'glitter_pages'
    verbose_name = 'Pages'

    def ready(self):
        super(PagesConfig, self).ready()
        from . import listeners  # noqa



# -*- coding: utf-8 -*-

from __future__ import unicode_literals


POSITION_IMAGE_LEFT = 'image-left'
POSITION_IMAGE_RIGHT = 'image-right'


POSITION_CHOICES = (
    (POSITION_IMAGE_LEFT, 'Image Left'),
    (POSITION_IMAGE_RIGHT, 'Image Right'),
)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible

from glitter.assets.fields import AssetForeignKey
from glitter.fields import LinkField
from glitter.models import BaseBlock


@python_2_unicode_compatible
class Banner(models.Model):
    title = models.CharField(max_length=100, db_index=True)
    image = AssetForeignKey('glitter_assets.Image', null=True, blank=True)
    description = models.TextField(blank=True)
    link = LinkField()
    link_text = models.CharField(max_length=100, blank=True)
    new_window = models.BooleanField('Open link in new window', default=False)

    class Meta:
        ordering = ('title',)

    def __str__(self):
        return self.title


class BannerBlock(BaseBlock):
    render_function = 'glitter.blocks.banner.views.banner_view'

    class Meta:
        verbose_name = 'banner'


@python_2_unicode_compatible
class BannerInline(models.Model):
    banner_block = models.ForeignKey(BannerBlock)
    banner = models.ForeignKey(Banner, on_delete=models.PROTECT)

    class Meta:
        verbose_name = 'banner'
        ordering = ('id',)

    def __str__(self):
        return '%s' % (self.banner,) or 'Banner'



# -*- coding: utf-8 -*-

from django import forms
from django.conf import settings

from glitter import block_admin
from .models import Redactor


class BaseRedactorForm(forms.ModelForm):
    class Meta:
        model = Redactor
        exclude = ()


def redactor_block_form():
    class_choices = getattr(settings, 'GLITTER_REDACTOR_CHOICES', ())

    # No classes? Just return a form without them
    if not class_choices:
        class RedactorForm(BaseRedactorForm):
            class Meta(BaseRedactorForm.Meta):
                exclude = ('block_class',)

        return RedactorForm

    # The more customisable version
    class_required = getattr(settings, 'GLITTER_REDACTOR_REQUIRED', False)

    class RedactorForm(BaseRedactorForm):
        block_class = forms.ChoiceField(
            label='Class', choices=class_choices, required=class_required)

    return RedactorForm


class RedactorAdmin(block_admin.BlockModelAdmin):
    form = redactor_block_form()


block_admin.site.register(Redactor, RedactorAdmin)
block_admin.site.register_block(Redactor, 'Common')



from django.core.exceptions import ValidationError
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from mptt.fields import TreeForeignKey

from glitter.fields import LinkField
from glitter.pages.models import Page
from glitter.models import BaseBlock


class RelatedPagesBlock(BaseBlock):
    title = models.CharField(max_length=100, blank=True, help_text='Defaults to "Related pages"')

    render_function = 'glitter.blocks.related_pages.views.relatedpages_view'

    class Meta:
        verbose_name = 'related pages'


@python_2_unicode_compatible
class RelatedPage(models.Model):
    related_pages_block = models.ForeignKey(RelatedPagesBlock)
    title = models.CharField(
        max_length=100, blank=True, help_text='Optional for pages, required for links')
    page = TreeForeignKey(Page, blank=True, null=True)
    link = LinkField(blank=True)
    position = models.PositiveIntegerField(default=0, db_index=True)

    class Meta:
        ordering = ('position',)

    def __str__(self):
        return str(self.page)

    def clean(self):
        if not self.page and not self.link:
            raise ValidationError('Need a page or a link')

        if self.page and self.link:
            raise ValidationError('Need a page or a link, not both')

        if self.link and not self.title:
            raise ValidationError('Need a title for a link')



# -*- coding: utf-8 -*-

from django.contrib.admin.sites import AdminSite
from django.contrib.admin.widgets import RelatedFieldWidgetWrapper
from django.core.urlresolvers import reverse
from django.forms.widgets import Select


class AddBlockSelect(Select):
    def render_options(self, choices, selected_choices):
        header = '<option value="" disabled="" selected="">Add a block:</option>'
        options = super(AddBlockSelect, self).render_options(choices, selected_choices)
        return header + options


class ChooseColumnSelect(Select):
    def render_options(self, choices, selected_choices):
        header = '<option value="" disabled="">Choose column:</option>'
        options = super(ChooseColumnSelect, self).render_options(choices, selected_choices)
        return header + options


class MoveBlockSelect(Select):
    def render_options(self, choices, selected_choices):
        header = '<option value="" disabled="" selected="">Move block:</option>'
        options = super(MoveBlockSelect, self).render_options(choices, selected_choices)
        return header + options


class CustomRelatedFieldWidgetWrapper(RelatedFieldWidgetWrapper):
    def get_related_url(self, info, action, *args):
        return reverse(
            "admin:%s_%s_%s" % (info + (action,)), current_app=AdminSite(), args=args
        )



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('glitter', '0001_initial'),
        ('glitter_pages', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='RelatedPage',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=100, help_text='Optional for pages, required for links', blank=True)),
                ('link', models.URLField(blank=True, null=True)),
                ('position', models.PositiveIntegerField(default=0, db_index=True)),
                ('page', mptt.fields.TreeForeignKey(blank=True, null=True, to='glitter_pages.Page')),
            ],
            options={
                'ordering': ('position',),
            },
        ),
        migrations.CreateModel(
            name='RelatedPagesBlock',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=100, help_text='Defaults to "Related pages"', blank=True)),
                ('content_block', models.ForeignKey(null=True, editable=False, to='glitter.ContentBlock')),
            ],
            options={
                'verbose_name': 'related pages',
            },
        ),
        migrations.AddField(
            model_name='relatedpage',
            name='related_pages_block',
            field=models.ForeignKey(to='glitter_related_pages.RelatedPagesBlock'),
        ),
    ]



# -*- coding: utf-8 -*-
from importlib import import_module

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.mail import EmailMessage
from django.core.urlresolvers import get_mod_func
from django.forms import ModelForm
from django.forms.fields import FileField
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils import six

from glitter.exceptions import GlitterRedirectException

from .models import BaseFormBlock, BaseFormNoEmailBlock
from .signals import form_valid


def form_view(block, request, rerender, content_block, block_classes, form_class=None):
    css_classes = ' '.join(block_classes)

    # Get the form class from the model
    if form_class is None:
        form_class = getattr(type(block), 'form_class', None)

    # Get the callable version of the form
    if isinstance(form_class, six.string_types):
        mod_name, class_name = get_mod_func(form_class)
        form_class = getattr(import_module(mod_name), class_name)

    if form_class is None:
        raise ImproperlyConfigured(
            "Form block %s requires a form_class attribute, or the render_function for %s needs"
            " to give a form_class" % (block, block))

    if isinstance(block, BaseFormBlock) and (not block.recipient or not block.success_page):
        # Need a recipient and a page to redirect to - so can't display form
        form = None
    elif isinstance(block, BaseFormNoEmailBlock) and not block.success_page:
        # Just need a success page - so can't display form
        form = None
    elif rerender:
        # Don't process a form on a rerender
        form = form_class()
    else:
        # All should be okay to process the form!
        form = form_class(request.POST or None, request.FILES or None)

        if form.is_valid():
            version = content_block.obj_version
            obj = version.content_object

            # Fire a signal if more functionality is needed
            form_valid.send(
                sender=form_class, request=request, form=form, obj=obj, version=version)

            # Save any model forms
            if isinstance(form, ModelForm):
                model_obj = form.save(commit=False)

                # Allow object tweaking if needed
                if hasattr(model_obj, 'pre_save'):
                    model_obj.pre_save(request=request)

                model_obj.save()

            # Don't send emails for this form block
            if isinstance(block, BaseFormNoEmailBlock):
                raise GlitterRedirectException(block.success_page.url)

            # Additional information for emailing
            page_url = '%s://%s%s' % (request.scheme, request.get_host(), obj.get_absolute_url())

            email_subject = settings.EMAIL_SUBJECT_PREFIX + block._meta.verbose_name
            email_body = render_to_string('glitter/form_email.txt', {
                'form': form,
                'obj': obj,
                'page_url': page_url,
            })
            email = EmailMessage(
                subject=email_subject,
                body=email_body,
                to=[block.recipient])

            # Add any attachments
            if form.is_multipart():
                for field in form:
                    if isinstance(field.field, FileField):
                        file_upload = form.cleaned_data[field.name]

                        if file_upload:
                            file_upload.open('rb')
                            file_content = file_upload.read()
                            email.attach(filename=file_upload.name, content=file_content)

            email.send(fail_silently=False)
            raise GlitterRedirectException(block.success_page.url)

    return render_to_string((
        'glitter/blocks/%s.html' % (block._meta.model_name,),
        'glitter/blocks/formblock.html',
    ), {
        'content_block': content_block,
        'css_classes': css_classes,
        'object': block,
        'form': form,
    }, context_instance=RequestContext(request))



# -*- coding: utf-8 -*-

from glitter import block_admin

from .forms import LatestTweetsBlockForm
from .models import LatestTweetsBlock


class LatestTweetsBlockAdmin(block_admin.BlockModelAdmin):
    form = LatestTweetsBlockForm


block_admin.site.register(LatestTweetsBlock, LatestTweetsBlockAdmin)
block_admin.site.register_block(LatestTweetsBlock, 'App Blocks')



# -*- coding: utf-8 -*-

from glitter import block_admin

from .models import SampleModelWithInlinesBlock, SampleInline


class SampleInlineAdmin(block_admin.StackedInline):
    model = SampleInline


class SampleModelWithInlinesBlockAdmin(block_admin.BlockModelAdmin):
    inlines = [SampleInlineAdmin]


block_admin.site.register(SampleModelWithInlinesBlock, SampleModelWithInlinesBlockAdmin)
block_admin.site.register_block(SampleModelWithInlinesBlock, 'Sample Blocks')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.encoding import python_2_unicode_compatible


@python_2_unicode_compatible
class Version(models.Model):
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')
    version_number = models.PositiveIntegerField(db_index=True, null=True)
    template_name = models.CharField(max_length=70)
    created = models.DateTimeField(auto_now_add=True, db_index=True)
    modified = models.DateTimeField(auto_now=True, db_index=True)
    owner = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.PROTECT, null=True)

    class Meta:
        ordering = ('-version_number',)
        unique_together = (
            ('content_type', 'object_id', 'version_number'),
        )

    def __str__(self):
        if self.version_number:
            return 'Version %d' % (self.version_number,)
        else:
            return 'Unpublished version %s' % (self.created,)

    def generate_version(self):
        # Create a version number if the page doesn't have one already
        if not self.version_number:
            try:
                prev_version = Version.objects.filter(
                    content_type=self.content_type, object_id=self.object_id).exclude(
                    version_number__isnull=True)[0].version_number
            except IndexError:
                prev_version = 0

            self.version_number = prev_version + 1

        return self.version_number

    @property
    def is_published(self):
        obj = self.content_object

        if self.version_number and obj.published and obj.current_version == self:
            return True
        else:
            return False


@python_2_unicode_compatible
class ContentBlock(models.Model):
    obj_version = models.ForeignKey(Version)
    column = models.CharField(max_length=100, db_index=True)
    position = models.IntegerField(null=True, db_index=True)
    content_type = models.ForeignKey(ContentType, related_name='+')
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')

    class Meta:
        ordering = ('position',)
        unique_together = (
            ('obj_version', 'column', 'position'),
        )

    def __str__(self):
        return self.content_type.name

    def save(self, *args, **kwargs):
        # Set the position to the highest possible if there isn't one already
        if self.position is None:
            last_position = ContentBlock.objects.filter(
                obj_version=self.obj_version, column=self.column).aggregate(
                models.Max('position'))['position__max']

            # Just incase it's the first block in this column
            if last_position is None:
                self.position = 1
            else:
                self.position = last_position + 1

        super(ContentBlock, self).save(*args, **kwargs)


class BaseBlock(models.Model):
    content_block = models.ForeignKey(ContentBlock, null=True, editable=False)

    # Override if more complex view logic is needed
    render_function = 'glitter.block_views.baseblock'

    class Meta:
        abstract = True



# -*- coding: utf-8 -*-

from django.db import models

from glitter.models import BaseBlock


class LatestTweetsBlock(BaseBlock):
    user = models.CharField(max_length=15, blank=True)

    class Meta:
        verbose_name = 'latest tweets'



# -*- coding: utf-8 -*-

from django.db.models.signals import pre_save, post_delete
from django.core.files.storage import default_storage

from .models import Image, File


def asset_file_change(sender, instance, **kwargs):
    # Must be saved already
    if instance.pk is not None:
        try:
            old_obj = sender.objects.get(pk=instance.pk)

            # Delete the old file if the file names don't match
            if old_obj.file.path != instance.file.path:
                default_storage.delete(old_obj.file.path)
        except:
            pass


def asset_file_delete(sender, instance, **kwargs):
    # Try and remove the file if possible
    try:
        instance.file.delete(save=False)
    except:
        pass


pre_save.connect(asset_file_change, sender=Image)
pre_save.connect(asset_file_change, sender=File)
post_delete.connect(asset_file_delete, sender=Image)
post_delete.connect(asset_file_delete, sender=File)



# -*- coding: utf-8 -*-

from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.db.models import Q
from django.http import Http404
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.encoding import force_text
from django.views.decorators.csrf import csrf_protect

from glitter.models import Version
from glitter.page import Glitter


@csrf_protect
def render_page(request, page, page_version, edit=False):
    glitter = Glitter(page_version, request=request)
    columns = glitter.render(edit_mode=edit)

    return render_to_response(page_version.template_name, {
        'glitter': glitter,
        'edit_mode': edit,
        'columns': columns,
        page._meta.model_name: page,
        'object': page,
    }, context_instance=RequestContext(request))


@csrf_protect
def render_object_unpublished(request, obj):
    info = obj._meta.app_label, obj._meta.model_name
    verbose_name = force_text(obj._meta.verbose_name)

    # Users without the edit permission for this object get a 404 instead of object not published
    if not request.user.has_perm('%s.edit_%s' % info):
        raise Http404('No published %(verbose_name)s available' % {
            'verbose_name': verbose_name,
        })

    # Find the latest version a user has access to
    version = Version.objects.filter(
        content_type=ContentType.objects.get_for_model(obj), object_id=obj.id).exclude(
        ~Q(owner=request.user), version_number__isnull=True).first()

    if version is not None:
        next_url = reverse('admin:%s_%s_version' % info, kwargs={'version_id': version.id})
    else:
        # No version available, go edit a new template
        next_url = reverse('admin:%s_%s_template' % info, kwargs={'object_id': obj.id})

    response = render_to_response('admin/glitter/object_unpublished.html', {
        'object': obj,
        'version': version,
        'next_url': next_url,
        'verbose_name': verbose_name,
    }, context_instance=RequestContext(request))

    return response



# -*- coding: utf-8 -*-

from django.utils.module_loading import autodiscover_modules


default_app_config = 'glitter.apps.GlittersConfig'


def autodiscover():
    """ Auto discover for layouts. """
    autodiscover_modules('layouts')



# -*- coding: utf-8 -*-


class GlitterRedirectException(Exception):
    def __init__(self, url):
        self.url = url

    def __str__(self):
        return 'Redirect to %s required' % (self.url,)


class GlitterUnpublishedException(Exception):
    def __init__(self, obj):
        self.obj = obj

    def __str__(self):
        return '%s is not published' % (self.obj,)



# -*- coding: utf-8 -*-

from django import template
from django.template.loader import get_template, select_template

register = template.Library()


@register.simple_tag(takes_context=True)
def glitter_head(context):
    """
    Template tag which renders the glitter CSS and JavaScript. Any resources
    which need to be loaded should be added here. This is only shown to users
    with permission to edit the page.
    """
    user = context.get('user')

    if user is not None and user.is_staff:
        template = get_template('glitter/include/head.html')
        return template.render(context)
    return ''


@register.simple_tag(takes_context=True)
def glitter_startbody(context):
    """
    Template tag which renders the glitter overlay and sidebar. This is only
    shown to users with permission to edit the page.
    """
    user = context.get('user')

    if user is not None and user.is_staff:
        template_list = ['glitter/include/startbody.html']

        # We've got a page with a glitter object:
        # - May need a different startbody template
        # - Check if user has permission to add
        glitter = context.get('glitter')
        if glitter is not None:
            opts = glitter.obj._meta.app_label, glitter.obj._meta.model_name
            template_list = [
                'glitter/include/startbody_%s_%s.html' % opts] + template_list

            context['has_add_permission'] = user.has_perm('%s.%s' % opts)

        template = select_template(template_list)
        return template.render(context)
    return ''



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='File',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, verbose_name='ID', auto_created=True)),
                ('title', models.CharField(db_index=True, max_length=100)),
                ('file', models.FileField(upload_to='assets/file')),
                ('file_size', models.PositiveIntegerField(default=0, editable=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('modified_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
                'ordering': ('-created_at', '-modified_at', 'title'),
            },
        ),
        migrations.CreateModel(
            name='FileCategory',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, verbose_name='ID', auto_created=True)),
                ('title', models.CharField(unique=True, max_length=100)),
            ],
            options={
                'verbose_name_plural': 'file categories',
                'abstract': False,
                'ordering': ('title',),
            },
        ),
        migrations.CreateModel(
            name='Image',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, verbose_name='ID', auto_created=True)),
                ('title', models.CharField(db_index=True, max_length=100)),
                ('file', models.ImageField(width_field='image_width', upload_to='assets/image', verbose_name='Image', height_field='image_height')),
                ('image_height', models.PositiveIntegerField(editable=False)),
                ('image_width', models.PositiveIntegerField(editable=False)),
                ('file_size', models.PositiveIntegerField(default=0, editable=False)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('modified_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
                'ordering': ('-created_at', '-modified_at', 'title'),
            },
        ),
        migrations.CreateModel(
            name='ImageCategory',
            fields=[
                ('id', models.AutoField(serialize=False, primary_key=True, verbose_name='ID', auto_created=True)),
                ('title', models.CharField(unique=True, max_length=100)),
            ],
            options={
                'verbose_name_plural': 'image categories',
                'abstract': False,
                'ordering': ('title',),
            },
        ),
        migrations.AddField(
            model_name='image',
            name='category',
            field=models.ForeignKey(to='glitter_assets.ImageCategory'),
        ),
        migrations.AddField(
            model_name='file',
            name='category',
            field=models.ForeignKey(to='glitter_assets.FileCategory'),
        ),
    ]



# -*- coding: utf-8 -*-

from os.path import basename

from django.contrib import admin

from .models import ImageCategory, Image, FileCategory, File


@admin.register(ImageCategory, FileCategory)
class CategoryAdmin(admin.ModelAdmin):
    search_fields = ('title',)


@admin.register(Image, File)
class FileAdmin(admin.ModelAdmin):
    date_hierarchy = 'created_at'
    list_display = ('title', 'category', 'file_link', 'modified_at', 'created_at',)
    list_filter = ('category',)
    search_fields = ('title',)
    readonly_fields = ('modified_at', 'created_at',)

    def file_link(self, obj):
        return '<a href="%s">%s</a>' % (obj.get_absolute_url(), basename(obj.file.name))
    file_link.short_description = 'Link'
    file_link.allow_tags = True



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import glitter.fields


class Migration(migrations.Migration):

    dependencies = [
        ('glitter_related_pages', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='relatedpage',
            name='link',
            field=glitter.fields.LinkField(default='', max_length=254, blank=True),
            preserve_default=False,
        ),
    ]



from django.core.exceptions import ValidationError
from django.core.validators import validate_email, URLValidator
from django.db.models.fields import CharField
from django.forms import URLField, CharField as CharFormField


url_validator = URLValidator()


def valid_mailto(value):
    # Must at least start with mailto
    if not value.lower().startswith('mailto:'):
        return False

    _, email = value.split(':', 1)

    try:
        validate_email(email)
        return True
    except ValidationError:
        return False


def valid_url(value):
    try:
        url_validator(value)
        return True
    except ValidationError:
        return False


def validate_link(value):
    """ Validate if a value is a valid link - either a URL or mailto. """
    if not any([valid_mailto(value), valid_url(value)]):
        raise ValidationError('Enter a valid link')


class LinkField(CharField):
    default_validators = [validate_link]
    description = 'Link'

    def __init__(self, *args, **kwargs):
        # max_length=254 to be compliant with RFCs 3696 and 5321
        kwargs['max_length'] = kwargs.get('max_length', 254)
        super(LinkField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        # As with CharField, this will cause validation to be performed twice.
        defaults = {
            'form_class': LinkFormField,
        }
        defaults.update(kwargs)
        return super(LinkField, self).formfield(**defaults)


class LinkFormField(URLField):
    default_error_messages = {
        'invalid': 'Enter a valid link',
    }
    default_validators = [validate_link]

    def to_python(self, value):
        # Slightly evil - ignore URLField's to_python if we're starting with mailto:
        value = super(CharFormField, self).to_python(value)

        if value and value.lower().startswith('mailto:'):
            return value

        # Go through URLField's to_python
        value = super(LinkFormField, self).to_python(value)

        return value



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('glitter', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='LatestTweetsBlock',
            fields=[
                ('id', models.AutoField(serialize=False, verbose_name='ID', primary_key=True, auto_created=True)),
                ('user', models.CharField(blank=True, max_length=15)),
                ('content_block', models.ForeignKey(null=True, to='glitter.ContentBlock', editable=False)),
            ],
            options={
                'verbose_name': 'latest tweets',
            },
        ),
    ]



# -*- coding: utf-8 -*-

from django.contrib import admin
from django.forms.widgets import Select

from glitter import block_admin
from glitter.widgets import CustomRelatedFieldWidgetWrapper
from .models import Banner, BannerBlock, BannerInline


@admin.register(Banner)
class BannerAdmin(admin.ModelAdmin):
    pass


class BannerInlineAdmin(block_admin.StackedInline):
    model = BannerInline
    extra = 1

    def formfield_for_dbfield(self, db_field, **kwargs):
        formfield = super(BannerInlineAdmin, self).formfield_for_dbfield(db_field, **kwargs)
        if db_field.name == 'banner':
            formfield.widget = CustomRelatedFieldWidgetWrapper(
                widget=Select(),
                rel=db_field.rel,
                admin_site=self.admin_site,
                can_add_related=True,
                can_change_related=True,
            )
        return formfield


class BannerBlockAdmin(block_admin.BlockModelAdmin):
    inlines = [
        BannerInlineAdmin,
    ]


block_admin.site.register(BannerBlock, BannerBlockAdmin)
block_admin.site.register_block(BannerBlock, 'App Blocks')



# -*- coding: utf-8 -*-

import os

from django.utils.crypto import get_random_string


BASE_DIR = os.path.dirname(__file__)


DEBUG = True


DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': 'test',
    }
}

CACHES = {
    'default': {
        'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
    }
}

SITE_ID = 1

STATIC_URL = '/static/'

SECRET_KEY = get_random_string(length=50)

ROOT_URLCONF = ''

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'glitter.pages.middleware.PageFallbackMiddleware',
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'glitter.tests.sample',
    'mptt',
    'glitter',
    'glitter.pages',
    'glitter.blocks.html',
    'glitter.tests.sampleblocks',
)

USE_TZ = True


APPEND_SLASH = True


GLITTER_LOGIN_PERMS = True


TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            os.path.join(BASE_DIR, 'templates'),
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.template.context_processors.request',
                'django.contrib.messages.context_processors.messages',
            ],
            'debug': True,
        },
    },
]



# -*- coding: utf-8 -*-

from django.db import router
from django.db.models.deletion import Collector
from django.db.models.fields.related import ForeignKey

from json import JSONEncoder


# Taken from simplejson, to avoid another dependency
class JSONEncoderForHTML(JSONEncoder):

    """An encoder that produces JSON safe to embed in HTML.

    To embed JSON content in, say, a script tag on a web page, the
    characters &, < and > should be escaped. They cannot be escaped
    with the usual entities (e.g. &amp;) because they are not expanded
    within <script> tags.
    """

    def encode(self, o):
        # Override JSONEncoder.encode because it has hacks for
        # performance that make things more complicated.
        chunks = self.iterencode(o, True)
        if self.ensure_ascii:
            return ''.join(chunks)
        else:
            return u''.join(chunks)

    def iterencode(self, o, _one_shot=False):
        chunks = super(JSONEncoderForHTML, self).iterencode(o, _one_shot)
        for chunk in chunks:
            chunk = chunk.replace('&', '\\u0026')
            chunk = chunk.replace('<', '\\u003c')
            chunk = chunk.replace('>', '\\u003e')
            yield chunk


# Dropping Collector from 1.5 to act like 1.4
class CloneCollector(Collector):
    def can_fast_delete(self, objs, from_field=None):
        return False


# Needed better duplication, this seems to be good!
# http://stackoverflow.com/a/6064096/584647
def duplicate(obj, value=None, field=None, duplicate_order=None):

    """
    Duplicate all related objects of obj setting
    field to value. If one of the duplicate
    objects has an FK to another duplicate object
    update that as well. Return the duplicate copy
    of obj.
    duplicate_order is a list of models which specify how
    the duplicate objects are saved. For complex objects
    this can matter. Check to save if objects are being
    saved correctly and if not just pass in related objects
    in the order that they should be saved.
    """
    using = router.db_for_write(obj._meta.model)
    collector = CloneCollector(using=using)
    collector.collect([obj])
    collector.sort()
    related_models = list(collector.data.keys())
    data_snapshot = {}
    for key in collector.data.keys():
        data_snapshot.update({
            key: dict(zip(
                [item.pk for item in collector.data[key]], [item for item in collector.data[key]]))
        })
    root_obj = None

    # Sometimes it's good enough just to save in reverse deletion order.
    if duplicate_order is None:
        duplicate_order = reversed(related_models)

    for model in duplicate_order:
        # Find all FKs on model that point to a related_model.
        fks = []
        for f in model._meta.fields:
            if isinstance(f, ForeignKey) and f.rel.to in related_models:
                fks.append(f)
        # Replace each `sub_obj` with a duplicate.
        if model not in collector.data:
            continue
        sub_objects = collector.data[model]
        for obj in sub_objects:
            for fk in fks:
                fk_value = getattr(obj, "%s_id" % fk.name)
                # If this FK has been duplicated then point to the duplicate.
                fk_rel_to = data_snapshot[fk.rel.to]
                if fk_value in fk_rel_to:
                    dupe_obj = fk_rel_to[fk_value]
                    setattr(obj, fk.name, dupe_obj)
            # Duplicate the object and save it.
            obj.id = None
            if field is not None:
                setattr(obj, field, value)
            obj.save()
            if root_obj is None:
                root_obj = obj
    return root_obj



from django.db import models


class BaseImageBlockManager(models.Manager):
    use_for_related_fields = True

    def get_queryset(self):
        qs = super(BaseImageBlockManager, self).get_queryset()
        return qs.select_related('image')



# -*- coding: utf-8 -*-

from django.contrib import admin
from django.forms.widgets import Select

from glitter import block_admin
from .models import (
    Carousel, CarouselImage, ImageOnlyCarousel, CarouselBlock, ImageOnlyCarouselBlock,
    ImageOnlyCarouselImage
)
from .widgets import CustomRelatedFieldWidgetWrapper


class CarouselImageInline(admin.StackedInline):
    model = CarouselImage
    extra = 1


class CarouselAdmin(admin.ModelAdmin):
    inlines = [CarouselImageInline]


class CarouselBlockAdmin(block_admin.BlockModelAdmin):
    def formfield_for_dbfield(self, db_field, **kwargs):
        formfield = super(CarouselBlockAdmin, self).formfield_for_dbfield(db_field, **kwargs)
        if db_field.name == 'carousel':
            formfield.widget = CustomRelatedFieldWidgetWrapper(
                widget=Select(),
                rel=db_field.rel,
                admin_site=self.admin_site,
                can_add_related=True,
                can_change_related=True,
            )
        return formfield


class ImageOnlyCarouselImageInline(admin.TabularInline):
    model = ImageOnlyCarouselImage
    extra = 1


class ImageOnlyCarouselAdmin(admin.ModelAdmin):
    inlines = [ImageOnlyCarouselImageInline]


class ImageOnlyCarouselBlockAdmin(block_admin.BlockModelAdmin):
    def formfield_for_dbfield(self, db_field, **kwargs):
        formfield = super(ImageOnlyCarouselBlockAdmin, self).formfield_for_dbfield(
            db_field, **kwargs)
        if db_field.name == 'carousel':
            formfield.widget = CustomRelatedFieldWidgetWrapper(
                widget=Select(),
                rel=db_field.rel,
                admin_site=self.admin_site,
                can_add_related=True,
                can_change_related=True,
            )
        return formfield


admin.site.register(Carousel, CarouselAdmin)
admin.site.register(ImageOnlyCarousel, ImageOnlyCarouselAdmin)

block_admin.site.register(CarouselBlock, CarouselBlockAdmin)
block_admin.site.register(ImageOnlyCarouselBlock, ImageOnlyCarouselBlockAdmin)

block_admin.site.register_block(CarouselBlock, 'Media')
block_admin.site.register_block(ImageOnlyCarouselBlock, 'Media')



# -*- coding: utf-8 -*-

from django.contrib.admin.sites import AdminSite
from django.contrib.admin.widgets import RelatedFieldWidgetWrapper
from django.core.urlresolvers import reverse


class CustomRelatedFieldWidgetWrapper(RelatedFieldWidgetWrapper):

    def get_related_url(self, info, action, *args):
        return reverse(
            "admin:%s_%s_%s" % (info + (action,)), current_app=AdminSite(), args=args
        )



# -*- coding: utf-8 -*-

from glitter import block_admin

from .models import DefinitionList, DefinitionListInline


class DefinitionListInlineAdmin(block_admin.TabularInline):
    model = DefinitionListInline


class DefinitionListAdmin(block_admin.BlockModelAdmin):
    inlines = [DefinitionListInlineAdmin]


block_admin.site.register(DefinitionList, DefinitionListAdmin)
block_admin.site.register_block(DefinitionList, 'Common')



# -*- coding: utf-8 -*-

from glitter import columns, templates
from glitter.layouts import PageLayout
from glitter.pages.models import Page


class SampleLayout(PageLayout):
    main_content = columns.Column(width=960)
    side = columns.Column(width=320)

    class Meta:
        template = 'glitter/sample.html'
        verbose_name = 'Sample'


class Sample2Layout(PageLayout):
    content = columns.Column(width=480)
    side = columns.Column(width=320)
    rside = columns.Column(width=320)

    class Meta:
        template = 'glitter/sample2.html'
        verbose_name = 'Sample2'


templates.register(SampleLayout, Page)
templates.register(Sample2Layout, 'glitter_pages.Page')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible

from glitter.mixins import GlitterMixin


@python_2_unicode_compatible
class Book(GlitterMixin):
    title = models.CharField(max_length=100)

    def __str__(self):
        return self.title



# -*- coding: utf-8 -*-

from django import forms
from django.db.models.fields import BLANK_CHOICE_DASH

from latest_tweets.models import Tweet

from .models import LatestTweetsBlock


class LatestTweetsBlockForm(forms.ModelForm):
    class Meta:
        model = LatestTweetsBlock
        exclude = ()

    user = forms.ChoiceField(required=False)

    def __init__(self, *args, **kwargs):
        super(LatestTweetsBlockForm, self).__init__(*args, **kwargs)

        # To get Tweet objects needs django-latest-tweets should be installed.
        self.fields['user'].choices = BLANK_CHOICE_DASH + [
            (x[0], '@%s - %s' % x) for x in Tweet.objects.order_by(
                'user').distinct().values_list('user', 'name')
        ]



# -*- coding: utf-8 -*-

from django.db import models

from glitter.exceptions import GlitterUnpublishedException
from glitter.page import Glitter

from .managers import GlitterManager


class GlitterMixin(models.Model):
    published = models.BooleanField(default=True, db_index=True)
    current_version = models.ForeignKey('glitter.Version', blank=True, null=True, editable=False)

    objects = GlitterManager()

    class Meta:
        default_permissions = ('add', 'change', 'delete', 'edit', 'publish')
        abstract = True


class GlitterDetailMixin(object):
    def post(self, request, *args, **kwargs):
        # By default detail views don't allow POST requests, however forms are usable as blocks.
        # So we allow POST requests, which does the same as GET.
        return self.get(request, *args, **kwargs)

    def get_object(self, queryset=None):
        obj = super(GlitterDetailMixin, self).get_object(queryset)

        version = self.kwargs.get('version')

        if version:
            self.glitter = Glitter(page_version=version, request=self.request)
        else:

            # If an object isn't viewable by end users - staff might still be able to edit the
            # object. Raise an exception and let middleware deal with it.
            if not obj.published or not obj.current_version:
                raise GlitterUnpublishedException(obj=obj)
            self.glitter = Glitter(page_version=obj.current_version, request=self.request)

        self.glitter_columns = self.glitter.render()

        return obj

    def get_template_names(self):
        return [self.glitter.version.template_name]

    def get_context_data(self, **kwargs):
        context = super(GlitterDetailMixin, self).get_context_data(**kwargs)

        obj = self.get_object()
        edit = self.kwargs.get('edit_mode')

        columns = self.glitter.render(edit_mode=edit)

        context['glitter'] = self.glitter
        context['columns'] = columns
        context['edit_mode'] = edit
        context[obj._meta.model_name] = obj
        return context



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import glitter.assets.fields
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('glitter', '0001_initial'),
        ('glitter_assets', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ImageBlock',
            fields=[
                ('id', models.AutoField(auto_created=True, serialize=False, verbose_name='ID', primary_key=True)),
                ('description', models.CharField(blank=True, help_text='Used as ALT text', max_length=200)),
                ('caption', models.CharField(blank=True, help_text='Shown below the image', max_length=200)),
                ('link', models.URLField(blank=True)),
                ('new_window', models.BooleanField(verbose_name='Open link in new window', default=False)),
                ('content_block', models.ForeignKey(null=True, to='glitter.ContentBlock', editable=False)),
                ('image', glitter.assets.fields.AssetForeignKey(on_delete=django.db.models.deletion.PROTECT, null=True, to='glitter_assets.Image')),
            ],
            options={
                'verbose_name': 'image',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import glitter.assets.fields


class Migration(migrations.Migration):

    dependencies = [
        ('glitter', '0001_initial'),
        ('glitter_assets', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='TextImageBlock',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, verbose_name='ID', serialize=False)),
                ('position', models.CharField(max_length=50)),
                ('content', models.TextField()),
                ('content_block', models.ForeignKey(to='glitter.ContentBlock', null=True, editable=False)),
                ('image', glitter.assets.fields.AssetForeignKey(to='glitter_assets.Image', null=True, on_delete=django.db.models.deletion.PROTECT)),
            ],
            options={
                'verbose_name': 'Text/Image',
            },
        ),
    ]



# -*- coding: utf-8 -*-

from django.db import models

from mptt.fields import TreeForeignKey

from glitter.models import BaseBlock


class BaseFormBlock(BaseBlock):
    recipient = models.EmailField()
    success_page = TreeForeignKey('glitter_pages.Page', null=True, on_delete=models.SET_NULL)

    render_function = 'glitter.blocks.form.views.form_view'

    class Meta:
        abstract = True


class BaseFormNoEmailBlock(BaseBlock):
    success_page = TreeForeignKey('glitter_pages.Page', null=True, on_delete=models.SET_NULL)

    render_function = 'glitter.blocks.form.views.form_view'

    class Meta:
        abstract = True


class ContactFormBlock(BaseFormBlock):
    form_class = 'glitter.blocks.form.forms.ContactForm'

    class Meta:
        verbose_name = 'contact form'



from django.contrib.contenttypes.models import ContentType
from django.test import TestCase

from glitter.blocks.html.models import HTML
from glitter.models import ContentBlock, Version
from glitter.page import Glitter
from glitter.pages.models import Page


class TestGlitterQueries(TestCase):
    def setUp(self):
        self.page = Page.objects.create(url='/test/', title='Test page')
        self.page_version = Version.objects.create(
            content_type=ContentType.objects.get_for_model(Page),
            object_id=self.page.id,
            template_name='glitter/sample.html',
        )

    def test_html_blocks(self):
        # Add 100 HTML blocks to a page
        html_content_type = ContentType.objects.get_for_model(HTML)

        for block_position in range(1, 101):
            html_block = HTML.objects.create(content='<p>HTML Block</p>')
            content_block = ContentBlock.objects.create(
                obj_version=self.page_version,
                column='main_content',
                position=block_position,
                content_type=html_content_type,
                object_id=html_block.id,
            )
            html_block.content_block = content_block
            html_block.save(update_fields=['content_block'])

        # Two queries:
        # - One to select content blocks, along with their content types
        # - Another to select all HTML blocks
        with self.assertNumQueries(2):
            glitter = Glitter(page_version=self.page_version)
            glitter.render()



# -*- coding: utf-8 -*-

from collections import OrderedDict
import inspect

from django.forms.forms import pretty_name
from django.template.defaultfilters import title
from django.utils import six
from django.utils.text import camel_case_to_spaces

from .columns import Column


class Options(object):
    def __init__(self, meta, cls):
        self.template = ''
        self.verbose_name = ''
        self.columns = {}

        self._update_template(cls, meta)
        self._update_verbose_name(cls, meta)

    def _update_template(self, cls, meta):
        if hasattr(meta, 'template'):
            self.template = meta.template
        else:
            self.template = 'glitter/{}.html'.format(cls.__name__.lower())

    def _update_verbose_name(self, cls, meta):
        if hasattr(meta, 'verbose_name'):
            self.verbose_name = meta.verbose_name
        else:
            self.verbose_name = title(camel_case_to_spaces(cls.__name__))


class PageLayoutBase(type):
    """
    Base Layout meta class used to store template structure details.
    """

    def __new__(cls, name, bases, attrs):
        super_new = super(PageLayoutBase, cls).__new__

        # (excluding PageLayout class itself).
        # Also ensure initialization is only performed for subclasses of PageLayout
        parents = [b for b in bases if isinstance(b, PageLayoutBase)]
        if not parents:
            new_class = super_new(cls, name, bases, attrs)
            return new_class

        attr_meta = attrs.pop('Meta', None)
        module = attrs.pop('__module__')
        new_class = super_new(cls, name, bases, {'__module__': module})
        if not attr_meta:
            meta = getattr(new_class, 'Meta', None)
        else:
            meta = attr_meta

        # Create meta API.
        meta = Options(meta, new_class)

        # If columns are inherited merge them with new colums.
        if hasattr(new_class, '_meta'):
            if new_class._meta.columns:
                attrs.update(new_class._meta.columns)

        if issubclass(new_class, PageLayout):
            # Columns
            columns = {}
            for column, cls in six.iteritems(attrs):
                if isinstance(cls, Column):
                    columns.setdefault(column, cls)

            columns = sorted(columns.items(), key=lambda x: getattr(x[1], 'creation_counter'))

            meta.columns = OrderedDict(columns)

        new_class.add_to_class('_meta', meta)

        return new_class

    def add_to_class(cls, name, value):
        # We should call the contribute_to_class method only if it's bound
        if not inspect.isclass(value) and hasattr(value, 'contribute_to_class'):
            value.contribute_to_class(cls, name)
        else:
            setattr(cls, name, value)

    def get_column_name(self, column_name):
        """ Get a column for given column name from META api. """
        name = pretty_name(column_name)
        if column_name in self._meta.columns:
            column_cls = self._meta.columns[column_name]
            if column_cls.verbose_name:
                name = column_cls.verbose_name
        return name


class PageLayout(six.with_metaclass(PageLayoutBase)):
    pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from django.http import HttpResponseRedirect

from .exceptions import GlitterRedirectException, GlitterUnpublishedException
from .views import render_object_unpublished


class ExceptionMiddleware(object):
    """
    The middleware glitter.middleware.ExceptionMiddleware is handles exceptions if the object
    doesn’t have the current version or hasn’t been published it prompts the user to create a new
    page it also deals with blocks which raise exception.
    """

    def process_exception(self, request, exception):
        if isinstance(exception, GlitterUnpublishedException):
            return render_object_unpublished(request=request, obj=exception.obj)

        if isinstance(exception, GlitterRedirectException):
            return HttpResponseRedirect(exception.url)

        return None



# -*- coding: utf-8 -*-

import os.path
from unittest import skipIf

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from django.core.urlresolvers import reverse
from django.conf import settings
from django.test import override_settings, modify_settings
from django.test.client import Client

from glitter.blocks.html.models import HTML
from glitter.pages.models import Page
from glitter.models import Version, ContentBlock


SAMPLE_BLOCK_MISSING = 'glitter.tests.sampleblocks' not in settings.INSTALLED_APPS
if not SAMPLE_BLOCK_MISSING:
    from glitter.tests.sampleblocks.models import (
        SampleModel, SampleModelWithInlinesBlock, SampleInline
    )


@modify_settings(INSTALLED_APPS={
    'append': 'glitter.tests.sampleblocks',
    'append': 'glitter.tests.sample',
})
@override_settings(
    MIDDLEWARE_CLASSES=(
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.middleware.common.CommonMiddleware',
        'django.middleware.csrf.CsrfViewMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django.contrib.auth.middleware.SessionAuthenticationMiddleware',
        'django.contrib.messages.middleware.MessageMiddleware',
        'glitter.pages.middleware.PageFallbackMiddleware',
    ),
    PASSWORD_HASHERS=(
        'django.contrib.auth.hashers.MD5PasswordHasher',
    ),
    TEMPLATE_DIRS=(
        os.path.join(os.path.dirname(__file__), 'templates'),
    ),
    ROOT_URLCONF='glitter.tests.urls',
)
class BaseEditCase(StaticLiveServerTestCase):

    def setUp(self):

        # Permissions
        edit_perm = Permission.objects.get_by_natural_key('edit_page', 'glitter_pages', 'page')
        publish_perm = Permission.objects.get_by_natural_key(
            'publish_page', 'glitter_pages', 'page'
        )

        # Content type used regularly
        self.page_content_type = ContentType.objects.get_for_model(Page)

        User = get_user_model()
        # Staff - admin access without pages permissions
        self.staff = User.objects.create_user(username='staff', password='staff')
        self.staff.is_staff = True
        self.staff.save()
        self.staff_client = Client()
        self.staff_client.login(username='staff', password='staff')

        # Editor - can edit pages but can't publish
        self.editor = User.objects.create_user(username='editor', password='editor')
        self.editor.is_staff = True
        self.editor.save()
        self.editor.user_permissions.add(edit_perm)
        self.editor_client = Client()
        self.editor_client.login(username='editor', password='editor')

        # Other editor, testing cases which protect users content
        self.editor2 = User.objects.create_user(username='editor2', password='editor2')
        self.editor2.is_staff = True
        self.editor2.save()
        self.editor2.user_permissions.add(edit_perm)
        self.editor2_client = Client()
        self.editor2_client.login(username='editor2', password='editor2')

        # Publisher - can edit and publish pages
        self.publisher = User.objects.create_user(username='publisher', password='publisher')
        self.publisher.is_staff = True
        self.publisher.save()
        self.publisher.user_permissions.add(edit_perm, publish_perm)
        self.publisher_client = Client()
        self.publisher_client.login(username='publisher', password='publisher')

        # Other publisher
        self.publisher2 = User.objects.create_user(username='publisher2', password='publisher2')
        self.publisher2.is_staff = True
        self.publisher2.save()
        self.publisher2.user_permissions.add(edit_perm, publish_perm)
        self.publisher2_client = Client()
        self.publisher2_client.login(username='publisher2', password='publisher2')

        # Anonymous user
        self.anon_client = Client()


class TestNoSavedVersions(BaseEditCase):
    def setUp(self):
        super(TestNoSavedVersions, self).setUp()

        # Simple page without any saved versions
        self.page_url = '/no-saved-versions/'
        Page.objects.create(url=self.page_url, title='Test page')

    def test_anon_user(self):
        # 404 for non-editors
        response = self.anon_client.get(self.page_url)
        self.assertEqual(response.status_code, 404)

    def test_editor(self):
        # 200 with link for editors
        response = self.editor_client.get(self.page_url)
        self.assertEqual(response.status_code, 200)


class TestCreateVersion(BaseEditCase):
    def setUp(self):
        super(TestCreateVersion, self).setUp()

        # Simple page without any saved versions
        page = Page.objects.create(url='/create-version/', title='Test page')

        # Change template URL
        info = page._meta.app_label, page._meta.model_name
        self.change_template_url = reverse('admin:%s_%s_template' % info, kwargs={
            'object_id': page.id,
        })

    def test_anon_user(self):
        # No for anon users
        response = self.anon_client.get(self.change_template_url, follow=True)
        self.assertTemplateUsed(response, 'admin/login.html')

    def test_staff(self):
        # No for staff
        response = self.staff_client.get(self.change_template_url)
        self.assertEqual(response.status_code, 403)

    def test_editor(self):
        # Editor can create a new version
        response = self.editor_client.get(self.change_template_url)
        self.assertTemplateUsed(response, 'admin/glitter/new_template.html')

        response = self.editor_client.post(self.change_template_url, {
            'template_name': 'glitter/sample.html',
        }, follow=True)
        self.assertEqual(response.status_code, 200)
        self.assertTemplateUsed(response, 'glitter/sample.html')


class TestViewVersion(BaseEditCase):
    def setUp(self):
        super(TestViewVersion, self).setUp()

        # Page with one unsaved version
        page = Page.objects.create(url='/view-version/', title='Test page')
        page_version = Version.objects.create(
            content_type=self.page_content_type, object_id=page.id,
            template_name='glitter/sample.html', owner=self.editor)

        # View page version URL
        info = page._meta.app_label, page._meta.model_name
        self.view_version_url = reverse('admin:%s_%s_version' % info, kwargs={
            'version_id': page_version.id,
        })

    def test_anon_user(self):
        # No for anon users
        response = self.anon_client.get(self.view_version_url, follow=True)
        self.assertTemplateUsed(response, 'admin/login.html')

    def test_staff(self):
        # No for staff
        response = self.staff_client.get(self.view_version_url)
        self.assertEqual(response.status_code, 403)

    def test_other_editor(self):
        # No for other editor
        response = self.editor2_client.get(self.view_version_url)
        self.assertEqual(response.status_code, 403)

    def test_editor(self):
        # Editor can view their own version
        response = self.editor_client.get(self.view_version_url)
        self.assertTemplateUsed(response, 'glitter/sample.html')


class TestRedirectVersion(BaseEditCase):
    def setUp(self):
        super(TestRedirectVersion, self).setUp()

        # Page with two unsaved versions
        self.page_url = '/redirect-version/'
        self.page = Page.objects.create(url=self.page_url, title='Test page')
        self.page_version_1 = Version.objects.create(
            content_type=self.page_content_type, object_id=self.page.id,
            template_name='glitter/sample.html', owner=self.editor)
        self.page_version_2 = Version.objects.create(
            content_type=self.page_content_type, object_id=self.page.id,
            template_name='glitter/sample.html', owner=self.editor2)

        # Redirect page URL
        info = self.page._meta.app_label, self.page._meta.model_name
        self.redirect_url = reverse('admin:%s_%s_redirect' % info, kwargs={
            'object_id': self.page.id,
        })

        # Editors URLs
        self.view_version_1_url = reverse('admin:%s_%s_version' % info, kwargs={
            'version_id': self.page_version_1.id,
        })
        self.view_version_2_url = reverse('admin:%s_%s_version' % info, kwargs={
            'version_id': self.page_version_2.id,
        })

        # New template URL
        self.new_template_url = reverse('admin:%s_%s_template' % info, kwargs={
            'object_id': self.page.id,
        })

    def test_anon_user(self):
        # No for anon users
        response = self.anon_client.get(self.redirect_url, follow=True)
        self.assertTemplateUsed(response, 'admin/login.html')

    def test_staff(self):
        # No for staff
        response = self.staff_client.get(self.redirect_url)
        self.assertEqual(response.status_code, 403)

    def test_other_editor(self):
        # Other editor gets asked to start with a new template (we're using the publisher though)
        response = self.publisher_client.get(self.redirect_url, follow=True)
        self.assertRedirects(response, self.new_template_url)
        self.assertTemplateUsed(response, 'admin/glitter/new_template.html')

    def test_editors(self):
        # Each editor gets redirected to their latest version
        response = self.editor_client.get(self.redirect_url)
        self.assertRedirects(response, self.view_version_1_url)

        response = self.editor2_client.get(self.redirect_url)
        self.assertRedirects(response, self.view_version_2_url)

    def test_published_page(self):
        # With a published page, editors always get redirected to it
        self.page_version_1.generate_version()
        self.page_version_1.save()
        self.page.current_version = self.page_version_1
        self.page.save()

        response = self.editor_client.get(self.redirect_url)
        self.assertRedirects(response, self.page_url)

        response = self.editor2_client.get(self.redirect_url)
        self.assertRedirects(response, self.page_url)


class TestEditVersion(BaseEditCase):
    def setUp(self):
        super(TestEditVersion, self).setUp()

        # Page with one unsaved version
        page = Page.objects.create(url='/edit-version/', title='Test page')
        self.page_version = Version.objects.create(
            content_type=self.page_content_type, object_id=page.id,
            template_name='glitter/sample.html', owner=self.editor)

        # View page version URL
        info = page._meta.app_label, page._meta.model_name
        self.edit_version_url = reverse('admin:%s_%s_edit' % info, kwargs={
            'version_id': self.page_version.id,
        })
        self.view_version_url = reverse('admin:%s_%s_version' % info, kwargs={
            'version_id': self.page_version.id,
        })

    def test_anon_user(self):
        # No for anon users
        response = self.anon_client.get(self.edit_version_url, follow=True)
        self.assertTemplateUsed(response, 'admin/login.html')

    def test_staff(self):
        # No for staff
        response = self.staff_client.get(self.edit_version_url)
        self.assertEqual(response.status_code, 403)

    def test_other_editor(self):
        # No for other editor
        response = self.editor2_client.get(self.edit_version_url)
        self.assertEqual(response.status_code, 403)

    def test_editor_unsaved_post(self):
        # Unsaved POST will redirect to the same URL
        response = self.editor_client.post(self.edit_version_url, {
            'template_name': 'glitter/sample.html',
        }, follow=True)
        self.assertRedirects(response, self.edit_version_url)
        self.assertTemplateUsed(response, 'glitter/sample.html')

    def test_editor_saved_get(self):
        # Save the version
        self.page_version.generate_version()
        self.page_version.save()

        # Get will redirect to the view URL
        response = self.editor_client.get(self.edit_version_url, follow=True)
        self.assertRedirects(response, self.view_version_url)
        self.assertTemplateUsed(response, 'glitter/sample.html')


@skipIf(SAMPLE_BLOCK_MISSING, 'glitter.tests.sampleblocks is not installed')
class TestEditCopyVersion(BaseEditCase):
    def setUp(self):
        super(TestEditCopyVersion, self).setUp()

        # Page with one saved version, an HTML block, and sample blocks
        page = Page.objects.create(url='/edit-version/', title='Test page')
        page_version = Version.objects.create(
            content_type=self.page_content_type, object_id=page.id,
            template_name='glitter/sample.html', owner=self.editor)

        html_block = HTML.objects.create(content='<p>HTML Block</p>')
        content_block = ContentBlock.objects.create(
            obj_version=page_version, column='content', position=1,
            content_type=ContentType.objects.get_for_model(html_block), object_id=html_block.id)
        html_block.content_block = content_block
        html_block.save()

        sample_model = SampleModel.objects.create(content='<p>Sample Model</p>')
        sample_block = SampleModelWithInlinesBlock.objects.create()
        SampleInline.objects.create(parent_block=sample_block, foreign_model=sample_model)
        content_block = ContentBlock.objects.create(
            obj_version=page_version, column='content', position=2,
            content_type=ContentType.objects.get_for_model(sample_block), object_id=sample_block.id)
        sample_block.content_block = content_block
        sample_block.save()

        page_version.generate_version()
        page_version.save()

        # Edit page URLs
        info = page._meta.app_label, page._meta.model_name
        self.edit_version1_url = reverse('admin:%s_%s_edit' % info, kwargs={
            'version_id': page_version.id,
        })
        self.edit_version2_url = reverse('admin:%s_%s_edit' % info, kwargs={
            'version_id': page_version.id + 1,
        })

    def test_edit_copy_version(self):
        # Generate a new page version
        response = self.editor_client.post(self.edit_version1_url, follow=True)
        self.assertRedirects(response, self.edit_version2_url)
        self.assertTemplateUsed(response, 'glitter/sample.html')


class TestSaveVersion(BaseEditCase):
    def setUp(self):
        super(TestSaveVersion, self).setUp()

        # Page with one unsaved version
        page = Page.objects.create(url='/save-version/', title='Test page')
        self.page_version = Version.objects.create(
            content_type=self.page_content_type, object_id=page.id,
            template_name='glitter/sample.html', owner=self.editor)

        # View page version URL
        info = page._meta.app_label, page._meta.model_name
        self.save_version_url = reverse('admin:%s_%s_save' % info, kwargs={
            'version_id': self.page_version.id,
        })
        self.view_version_url = reverse('admin:%s_%s_version' % info, kwargs={
            'version_id': self.page_version.id,
        })

    def test_anon_user(self):
        # No for anon users
        response = self.anon_client.post(self.save_version_url, follow=True)
        self.assertTemplateUsed(response, 'admin/login.html')

    def test_editor_get(self):
        # Have to POST to save
        response = self.editor_client.get(self.save_version_url)
        self.assertEqual(response.status_code, 405)

    def test_staff(self):
        # No for staff
        response = self.staff_client.post(self.save_version_url)
        self.assertEqual(response.status_code, 403)

    def test_other_editor(self):
        # No for other editor
        response = self.editor2_client.post(self.save_version_url)
        self.assertEqual(response.status_code, 403)

    def test_editor(self):
        # Generates a version number
        response = self.editor_client.post(self.save_version_url, follow=True)
        page_version = Version.objects.get(id=self.page_version.id)  # refetch
        self.assertNotEqual(page_version.version_number, None)
        self.assertRedirects(response, self.view_version_url)
        self.assertTemplateUsed(response, 'glitter/sample.html')

    def test_editor_saved_version(self):
        # Generate a saved version
        self.page_version.generate_version()
        self.page_version.save()

        # User will just be redirected to the version view page
        response = self.editor_client.post(self.save_version_url, follow=True)
        page_version = Version.objects.get(id=self.page_version.id)  # refetch
        self.assertEqual(self.page_version.version_number, page_version.version_number)
        self.assertRedirects(response, self.view_version_url)
        self.assertTemplateUsed(response, 'glitter/sample.html')


class TestPublishVersion(BaseEditCase):
    def setUp(self):
        super(TestPublishVersion, self).setUp()

        # Page with one unsaved version
        self.page_url = '/publish-version/'
        self.page = Page.objects.create(url=self.page_url, title='Test page')
        self.page_version = Version.objects.create(
            content_type=self.page_content_type, object_id=self.page.id,
            template_name='glitter/sample.html', owner=self.editor)

        # Edit page URLs
        info = self.page._meta.app_label, self.page._meta.model_name
        self.publish_version_url = reverse('admin:%s_%s_publish' % info, kwargs={
            'version_id': self.page_version.id,
        })

    def test_anon_user(self):
        # No for anon users
        response = self.anon_client.post(self.publish_version_url, follow=True)
        self.assertTemplateUsed(response, 'admin/login.html')

    def test_publisher_post(self):
        # Have to POST to publish
        response = self.editor_client.get(self.publish_version_url)
        self.assertEqual(response.status_code, 405)

    def test_staff(self):
        # No for staff
        response = self.staff_client.post(self.publish_version_url)
        self.assertEqual(response.status_code, 403)

    def test_other_publisher(self):
        # No for other publisher
        response = self.publisher2_client.post(self.publish_version_url)
        self.assertEqual(response.status_code, 403)

    def test_editor(self):
        # Editor without publish permission can't publish
        response = self.editor_client.post(self.publish_version_url)
        self.assertEqual(response.status_code, 403)

    def test_publish_unsaved(self):
        # Change the object owner for testing
        self.page_version.owner = self.publisher
        self.page_version.save()

        # Unsaved version publish saves a new version number, and redirects to the live URL
        response = self.publisher_client.post(self.publish_version_url, follow=True)
        page_version = Version.objects.get(id=self.page_version.id)  # refetch
        self.assertNotEqual(page_version.version_number, None)
        self.assertRedirects(response, self.page.get_absolute_url())
        self.assertTemplateUsed(response, 'glitter/sample.html')

    def test_publish_saved(self):
        self.page_version.owner = self.publisher
        self.page_version.generate_version()
        self.page_version.save()

        # Unsaved version publish saves a new version number, and redirects to the live URL
        response = self.publisher_client.post(self.publish_version_url, follow=True)
        self.assertRedirects(response, self.page.get_absolute_url())
        self.assertTemplateUsed(response, 'glitter/sample.html')

    def test_publish_current_version(self):
        self.page_version.owner = self.publisher
        self.page_version.generate_version()
        self.page_version.save()
        self.page.current_version = self.page_version
        self.page.save()

        # This should just redirect to the live URL
        response = self.publisher_client.post(self.publish_version_url, follow=True)
        page_version = Version.objects.get(id=self.page_version.id)  # refetch
        self.assertEqual(self.page_version.version_number, page_version.version_number)
        self.assertRedirects(response, self.page_url)
        self.assertTemplateUsed(response, 'glitter/sample.html')


class TestUnpublishVersion(BaseEditCase):
    def setUp(self):
        super(TestUnpublishVersion, self).setUp()

        # Page with one published version
        self.page_url = '/edit-version/'
        self.page = Page.objects.create(url=self.page_url, title='Test page')
        self.page_version = Version(
            content_type=self.page_content_type, object_id=self.page.id,
            template_name='glitter/sample.html', owner=self.editor)
        self.page_version.generate_version()
        self.page_version.save()
        self.page.current_version = self.page_version
        self.page.save()

        # Edit page URLs
        info = self.page._meta.app_label, self.page._meta.model_name
        self.unpublish_version_url = reverse('admin:%s_%s_unpublish' % info, kwargs={
            'version_id': self.page_version.id,
        })
        self.view_version_url = reverse('admin:%s_%s_version' % info, kwargs={
            'version_id': self.page_version.id,
        })

    def test_anon_user(self):
        # No for anon users
        response = self.anon_client.post(self.unpublish_version_url, follow=True)
        self.assertTemplateUsed(response, 'admin/login.html')

    def test_publisher_post(self):
        # Have to POST to unpublish
        response = self.editor_client.get(self.unpublish_version_url)
        self.assertEqual(response.status_code, 405)

    def test_staff(self):
        # No for staff
        response = self.staff_client.post(self.unpublish_version_url)
        self.assertEqual(response.status_code, 403)

    def test_editor(self):
        # Editor without publish permission can't unpublish
        response = self.editor_client.post(self.unpublish_version_url)
        self.assertEqual(response.status_code, 403)

    def test_unpublish(self):
        # We'll be redirected to the view version which is just unpublished
        response = self.publisher_client.post(self.unpublish_version_url, follow=True)
        page = Page.objects.get(id=self.page.id)  # refetch
        self.assertEqual(page.current_version, None)
        self.assertRedirects(response, self.view_version_url)
        self.assertTemplateUsed(response, 'glitter/sample.html')

    def test_unpublish_not_current(self):
        new_version = Version(
            content_type=self.page_content_type, object_id=self.page.id,
            template_name='glitter/sample.html', owner=self.editor)
        new_version.generate_version()
        new_version.save()
        self.page.current_version = new_version
        self.page.save()

        # Unsaved version publish saves a new version number, and redirects to the live URL
        response = self.publisher_client.post(self.unpublish_version_url, follow=True)
        page = Page.objects.get(id=self.page.id)  # refetch
        self.assertEqual(page.current_version.id, new_version.id)
        self.assertRedirects(response, self.view_version_url)
        self.assertTemplateUsed(response, 'glitter/sample.html')


class TestDiscardVersion(BaseEditCase):
    def setUp(self):
        super(TestDiscardVersion, self).setUp()

        # Page with one saved version, one unsaved version
        self.page_url = '/edit-version/'
        self.page = Page.objects.create(url=self.page_url, title='Test page')
        self.page_version_1 = Version(
            content_type=self.page_content_type, object_id=self.page.id,
            template_name='glitter/sample.html', owner=self.editor)
        self.page_version_1.generate_version()
        self.page_version_1.save()
        self.page.current_version = self.page_version_1
        self.page.save()
        self.page_version_2 = Version.objects.create(
            content_type=self.page_content_type, object_id=self.page.id,
            template_name='glitter/sample.html', owner=self.editor)

        # Unsaved version has an HTML block
        html_block = HTML.objects.create(content='<p>HTML Block</p>')
        content_block = ContentBlock.objects.create(
            obj_version=self.page_version_2, column='main_content', position=1,
            content_type=ContentType.objects.get_for_model(html_block), object_id=html_block.id)
        html_block.content_block = content_block
        html_block.save()

        # Discard page URLs
        info = self.page._meta.app_label, self.page._meta.model_name
        self.discard_version_1_url = reverse('admin:%s_%s_discard' % info, kwargs={
            'version_id': self.page_version_1.id,
        })
        self.discard_version_2_url = reverse('admin:%s_%s_discard' % info, kwargs={
            'version_id': self.page_version_2.id,
        })

    def test_anon_user(self):
        # No for anon users
        response = self.anon_client.post(self.discard_version_2_url, follow=True)
        self.assertTemplateUsed(response, 'admin/login.html')

    def test_staff(self):
        # No for staff
        response = self.staff_client.post(self.discard_version_2_url)
        self.assertEqual(response.status_code, 403)

    def test_saved_version(self):
        # Can't discard a saved version
        response = self.editor_client.post(self.discard_version_1_url)
        self.assertEqual(response.status_code, 403)

    def test_other_editor(self):
        # No for other editor
        response = self.editor2_client.post(self.discard_version_2_url)
        self.assertEqual(response.status_code, 403)

    def test_discard_get(self):
        # Discard popup loads a simple form
        response = self.editor_client.get(self.discard_version_2_url)
        self.assertTemplateUsed(response, 'admin/glitter/version_discard.html')

    def test_discard_post(self):
        # Version gets deleted
        response = self.editor_client.post(self.discard_version_2_url)
        self.assertRaises(
            Version.DoesNotExist, Version.objects.get, id=self.page_version_2.id)
        self.assertTemplateUsed(response, 'admin/glitter/version_discarded.html')



from django.db import models

from glitter.fields import LinkField
from glitter.models import BaseBlock


class CallToActionBlock(BaseBlock):
    title = models.CharField(max_length=100)
    link = LinkField()

    class Meta:
        verbose_name = 'call to action'



# -*- coding: utf-8 -*-

from collections import defaultdict, OrderedDict
from importlib import import_module

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import get_mod_func, reverse
from django.db.models import Q
from django.forms.widgets import Select
from django.template.defaultfilters import slugify
from django.template.loader import render_to_string
from django.utils import six
from django.utils.encoding import force_text
from django.utils.functional import cached_property
from django.utils.text import capfirst

from .models import Version
from .templates import get_layout, get_templates
from .widgets import AddBlockSelect, ChooseColumnSelect, MoveBlockSelect


# If no other settings are provided, just show HTML as a quick add block
GLITTER_DEFAULT_BLOCKS = (
    ('glitter.blocks.HTML', 'HTML'),
)


class GlitterBlock(object):
    def __init__(self, content_block, column, block_number):
        self.content_block = content_block
        self.block = content_block.content_object
        self.column = column
        self.glitter_page = column.glitter_page
        self.block_number = block_number

    def render(self, rerender):
        # Add some classes to the block to help style it
        block_classes = self.css_classes()

        # Render the block
        mod_name, func_name = get_mod_func(self.block.render_function)
        block_view = getattr(import_module(mod_name), func_name)
        self.html = block_view(
            self.block, self.glitter_page.request, rerender, self.content_block, block_classes)

    def css_classes(self):
        # Add some classes to the block to help style it
        block_classes = ['glitter_page_blocktype_%s' % (self.block._meta.model_name,)]

        if self.block_number == 1:
            block_classes.append('glitter_page_block_first')

        if self.block_number % 2 == 0:
            block_classes.append('glitter_page_block glitter_page_block_even')
        else:
            block_classes.append('glitter_page_block glitter_page_block_odd')

        if self.block_number == len(self.column.blocks):
            block_classes.append('glitter_page_block_last')

        return block_classes

    def block_type(self):
        return capfirst(force_text(self.content_block.content_object._meta.verbose_name))

    def edit_url(self):
        opts = self.block._meta.app_label, self.block._meta.model_name
        return reverse('block_admin:%s_%s_change' % opts, args=(self.block.pk,))

    def choose_column_widget(self):
        column_options = self.glitter_page.get_column_choices()

        widget = ChooseColumnSelect(attrs={
            'class': 'glitter-move-column-select',
        }, choices=column_options)

        return widget.render(name='', value=self.column.name)

    def move_block_widget(self):
        from glitter.forms import MoveBlockForm

        move_options = []

        if self.block_number != 1:
            move_options.extend((
                (MoveBlockForm.MOVE_TOP, 'To top'),
                (MoveBlockForm.MOVE_UP, 'Up'),
            ))

        if self.block_number != len(self.column.blocks):
            move_options.extend((
                (MoveBlockForm.MOVE_DOWN, 'Down'),
                (MoveBlockForm.MOVE_BOTTOM, 'To bottom'),
            ))

        widget = MoveBlockSelect(attrs={
            'class': 'glitter-move-block-select',
        }, choices=move_options)

        return widget.render(name='', value=None)


class GlitterColumn(object):
    def __init__(self, name, verbose_name, glitter_page, content_blocks):
        self.name = name
        self.verbose_name = verbose_name
        self.glitter_page = glitter_page
        self.blocks = []

        # Build up a list of page blocks
        for block_num, content_block in enumerate(content_blocks, start=1):
            block = GlitterBlock(content_block, self, block_num)
            self.blocks.append(block)

    def render(self, edit_mode=False, rerender=False):
        # Render all the blocks
        for block in self.blocks:
            block.render(rerender)

        # Column structure
        column_template = 'glitter/include/column.html'
        column_context = {
            'blocks': self.blocks,
            'column_slug': slugify(self.name),
        }

        if edit_mode:
            # Edit mode has a more complex template
            column_template = 'glitter/include/column_edit.html'

            column_context.update({
                'glitter': self.glitter_page,
                'column_name': self.name,
                'verbose_name': self.verbose_name,
                'default_blocks': getattr(
                    settings, 'GLITTER_DEFAULT_BLOCKS', GLITTER_DEFAULT_BLOCKS),
                'add_block_widget': self.add_block_widget(),
            })

        return render_to_string(column_template, column_context)

    def add_block_widget(self):
        widget = AddBlockSelect(attrs={
            'class': 'glitter-add-block-select',
        }, choices=self.add_block_options())

        return widget.render(name='', value=None)

    def add_block_options(self):
        from glitter import block_admin

        block_choices = []

        # Group all block by category
        for category in sorted(block_admin.site.block_list):
            category_blocks = block_admin.site.block_list[category]
            category_choices = (('%s.%s' % (x._meta.app_label, x._meta.object_name),
                                 capfirst(force_text(x._meta.verbose_name))) for x in
                                category_blocks)
            category_choices = sorted(category_choices, key=lambda x: x[1])
            block_choices.append((category, category_choices))

        return block_choices


class Glitter(object):
    """
    A page object which contains all of the data necessary to render a glitter template.
    """

    saved_pages = None
    unsaved_pages = None

    def __init__(self, page_version, request=None):
        self.version = page_version
        self.obj = page_version.content_object
        self.content_type = ContentType.objects.get_for_model(self.obj)
        self.opts = self.obj._meta
        self.request = request

        if request is None:
            self.user = None
        else:
            self.user = request.user

        self.layout = get_layout(template_name=page_version.template_name)

        # Only show controls to logged in users
        if self.user is not None:
            self.show_controls = self.user.has_perm(
                '%s.edit_%s' % (self.opts.app_label, self.opts.model_name))
        else:
            self.show_controls = False

        # Fetch all content blocks in one go
        self.column_blocks = defaultdict(list)
        content_blocks = self.version.contentblock_set.select_related(
            'content_type'
        ).prefetch_related('content_object').all()

        for content_block in content_blocks:
            self.column_blocks[content_block.column].append(content_block)

    def render(self, edit_mode=False, rerender=False):
        columns = OrderedDict()

        for column_name in self.layout._meta.columns:
            verbose_name = self.layout.get_column_name(column_name)

            column = GlitterColumn(
                name=column_name,
                verbose_name=verbose_name,
                glitter_page=self,
                content_blocks=self.column_blocks[column_name]
            )
            columns[column_name] = column.render(edit_mode=edit_mode, rerender=rerender)

        return columns

    def owner_versions(self):
        # Fiddly queryset which hides unsaved versions of other users
        return Version.objects.select_related('owner').filter(
            content_type=self.content_type, object_id=self.obj.id).exclude(
            ~Q(owner=self.user), version_number__isnull=True
        )

    def all_versions(self):
        return self.owner_versions().order_by('-modified')

    @cached_property
    def previous_version(self):
        try:
            return self.owner_versions().filter(
                modified__lt=self.version.modified).order_by('-modified')[0]
        except IndexError:
            return None

    @cached_property
    def next_version(self):
        try:
            return self.owner_versions().filter(
                modified__gt=self.version.modified
            ).order_by('modified')[0]
        except IndexError:
            return None

    def change_template_widget(self):
        change_template_options = sorted(get_templates(self.obj.__class__), key=lambda x: x[1])

        widget = Select(attrs={
            'id': 'id_template_name',
        }, choices=change_template_options)

        return widget.render(name='template_name', value=self.layout._meta.template)

    def get_column_choices(self):
        choices = []
        for column, cls in six.iteritems(self.layout._meta.columns):
            name = self.layout.get_column_name(column)
            choices.append((column, name))
        return choices



from django import template
from django.shortcuts import resolve_url

from glitter.pages.models import Page

register = template.Library()


@register.assignment_tag
def get_active_page(current_url):
    try:
        return Page.objects.get(url=resolve_url(current_url))
    except Page.DoesNotExist:
        pass

    return None


@register.assignment_tag
def get_root_pages(current_page=None):
    if current_page and not isinstance(current_page, Page):
        try:
            current_page = Page.objects.get(url=resolve_url(current_page))
        except Page.DoesNotExist:
            current_page = None

    page_list = []

    # Find the root page so the template can highlight it
    if current_page:
        root_page = current_page.get_root()
    else:
        root_page = None

    for i in Page.objects.root_nodes().filter(show_in_navigation=True):
        page_list.append((i, i == root_page))

    return page_list


@register.assignment_tag
def get_pages_at_level(current_page, level=1):
    if current_page and not isinstance(current_page, Page):
        try:
            current_page = Page.objects.get(url=resolve_url(current_page))
        except Page.DoesNotExist:
            current_page = None

    if not current_page:
        return []

    page_and_ancestors = list(current_page.get_ancestors(include_self=True))

    # Page isn't deep enough to show this level of navigation
    if level > current_page.level + 1:
        return []

    parent_page = page_and_ancestors[level - 1]
    page_list = []

    for i in parent_page.get_children().filter(show_in_navigation=True):
        page_list.append((i, i in page_and_ancestors))

    return page_list


@register.assignment_tag
def tree_from_root(current_page=None):
    page_model = Page
    tree = None

    if current_page and not isinstance(current_page, Page):
        try:
            current_page = page_model.objects.get(url=resolve_url(current_page))
        except page_model.DoesNotExist:
            current_page = None

    if current_page:
        root_page = current_page.get_root()
        tree = root_page.get_descendants()

    return tree


@register.assignment_tag
def get_page_ancestor_ids(current_page=None):
    page_model = Page
    ancestors = []

    if current_page and not isinstance(current_page, Page):
        try:
            current_page = page_model.objects.get(url=resolve_url(current_page))
        except page_model.DoesNotExist:
            current_page = None

    if current_page:
        ancestors = current_page.get_ancestors(include_self=True).values_list('id', flat=True)

    return ancestors



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='ContentBlock',
            fields=[
                ('id', models.AutoField(primary_key=True, verbose_name='ID', serialize=False, auto_created=True)),
                ('column', models.CharField(db_index=True, max_length=100)),
                ('position', models.IntegerField(db_index=True, null=True)),
                ('object_id', models.PositiveIntegerField()),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType', related_name='+')),
            ],
            options={
                'ordering': ('position',),
            },
        ),
        migrations.CreateModel(
            name='Version',
            fields=[
                ('id', models.AutoField(primary_key=True, verbose_name='ID', serialize=False, auto_created=True)),
                ('object_id', models.PositiveIntegerField()),
                ('version_number', models.PositiveIntegerField(db_index=True, null=True)),
                ('template_name', models.CharField(max_length=70)),
                ('created', models.DateTimeField(auto_now_add=True, db_index=True)),
                ('modified', models.DateTimeField(auto_now=True, db_index=True)),
                ('content_type', models.ForeignKey(to='contenttypes.ContentType')),
                ('owner', models.ForeignKey(to=settings.AUTH_USER_MODEL, null=True, on_delete=django.db.models.deletion.PROTECT)),
            ],
            options={
                'ordering': ('-version_number',),
            },
        ),
        migrations.AddField(
            model_name='contentblock',
            name='obj_version',
            field=models.ForeignKey(to='glitter.Version'),
        ),
        migrations.AlterUniqueTogether(
            name='version',
            unique_together=set([('content_type', 'object_id', 'version_number')]),
        ),
        migrations.AlterUniqueTogether(
            name='contentblock',
            unique_together=set([('obj_version', 'column', 'position')]),
        ),
    ]



# -*- coding: utf-8 -*-

from django.conf.urls import include, url
from django.contrib import admin
from glitter import block_admin


urlpatterns = [
    # Django admin
    url(r'^admin/', include(admin.site.urls)),

    # Glitter block admin
    url(r'^blockadmin/', include(block_admin.site.urls)),

]



# -*- coding: utf-8 -*-

import os

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.conf import settings
from django.test import TestCase, Client
from django.test import override_settings, modify_settings

from glitter.models import Version
from glitter.pages.models import Page


SAMPLE_BLOCK_MISSING = 'glitter.tests.sampleblocks' not in settings.INSTALLED_APPS


@modify_settings(
    INSTALLED_APPS={
        'append': 'glitter.tests.sample',
    },
)
@override_settings(
    PASSWORD_HASHERS=('django.contrib.auth.hashers.MD5PasswordHasher',),
    TEMPLATE_DIRS=(os.path.join(os.path.dirname(__file__), 'templates'),),
    ROOT_URLCONF='glitter.tests.urls',
)
class TestAdmin(TestCase):

    def setUp(self):

        # Permissions
        self.edit_permissions = Permission.objects.get_by_natural_key(
            'edit_page', 'glitter_pages', 'page'
        )

        # Page
        self.page = Page.objects.create(url='/test/', title='Test page', published=True)

        User = get_user_model()
        # Editor with editing permissions
        self.editor = User.objects.create_user('editor', 'editor@test.com', 'editor')
        self.editor.is_staff = True
        self.editor.user_permissions.add(self.edit_permissions)
        self.editor.save()
        self.editor_client = Client()
        self.editor_client.login(username='editor', password='editor')

        # Editor with not editing permissions
        self.editor_no_permissions = User.objects.create_user(
            'editor_no_perm', 'editor_no_perm@test.com', 'editor_no_perm'
        )
        self.editor_no_permissions.save()
        self.editor_no_permissions_client = Client()

        # Page version.
        self.page_version = Version.objects.create(
            content_type=ContentType.objects.get_for_model(Page), object_id=self.page.id,
            template_name='glitter/sample.html', owner=self.editor,
            version_number=1
        )

        self.page.current_version = self.page_version
        self.page.save()

    def test_glitter_head(self):
        """Test as front-end user to see if controllers are shown."""

        response = self.editor_no_permissions_client.get(self.page.url)
        self.assertEqual(response.status_code, 200)

        response = self.editor_client.get(self.page.url)
        self.assertEqual(response.status_code, 200)



# -*- coding: utf-8 -*-

from django.template.loader import render_to_string
from glitter.templates import get_layout


def baseblock(block, request, rerender, content_block, block_classes):
    css_classes = ' '.join(block_classes)

    # Find the column
    layout = get_layout(template_name=content_block.obj_version.template_name)
    column = layout._meta.columns[content_block.column]

    return render_to_string((
        'glitter/blocks/%s.html' % (block._meta.model_name,),
        'glitter/blocks/baseblock.html',
    ), {
        'content_block': content_block,
        'css_classes': css_classes,
        'object': block,
        'column': column,
    })



# -*- coding: utf-8 -*-

from django.conf import settings
from django.contrib.auth.decorators import user_passes_test
from django.http import Http404, HttpResponsePermanentRedirect
from django.shortcuts import get_object_or_404

from glitter.views import render_object_unpublished, render_page
from .models import Page


# Called from glitter.middleware.GlitterFallbackMiddleware
# Mostly a copy of the django.contrib.flatpages implementation
def glitter(request, url):
    PAGE_LOGIN_PERMS = getattr(settings, 'GLITTER_LOGIN_PERMS', False)

    if not url.startswith('/'):
        url = '/' + url

    page_opts = Page._meta.app_label, Page._meta.model_name

    try:
        page = get_object_or_404(
            Page.objects.select_related('current_version'), url__exact=url)
    except Http404:
        if not url.endswith('/') and settings.APPEND_SLASH:
            url += '/'
            page = get_object_or_404(Page, url__exact=url)
            return HttpResponsePermanentRedirect('%s/' % request.path)
        else:
            raise

    # Unpublished page
    if not page.published or not page.current_version:
        return render_object_unpublished(request=request, obj=page)

    # Page with login required
    if page.login_required:
        def can_view_page(user):
            if PAGE_LOGIN_PERMS:
                # Need the 'view_protected_page' permission to access this page
                return (user.has_perm('%s.view_protected_%s' % page_opts) or
                        user.has_perm('%s.view_protected_%s' % page_opts, obj=page))
            else:
                # Any authenticated user will do
                return user.is_authenticated()

        @user_passes_test(can_view_page)
        def view_protected_page(request):
            return render_page(request, page, page.current_version)

        # Pass it to the protected version to see if the user can view the page
        return view_protected_page(request)

    return render_page(request, page, page.current_version)



# -*- coding: utf-8 -*-

from django.http import Http404, HttpResponseRedirect

from glitter.exceptions import GlitterRedirectException, GlitterUnpublishedException


class PageFallbackMiddleware(object):
    def process_response(self, request, response):

        # Import here as it causes migrations for pages if app is not installed apps.
        from glitter.pages.views import glitter, render_object_unpublished

        if response.status_code != 404:
            return response  # No need to check for a page for non-404 responses.

        try:
            return glitter(request=request, url=request.path_info)
        except Http404:
            # Return the original response if a 404 occurs
            return response
        except GlitterUnpublishedException as exception:
            return render_object_unpublished(request=request, obj=exception.obj)
        except GlitterRedirectException as exception:
            return HttpResponseRedirect(exception.url)

        return None



# -*- coding: utf-8 -*-

import os

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.test import TestCase, Client, override_settings, modify_settings

from glitter.blocks.html.models import HTML
from glitter.models import Version, ContentBlock
from glitter.pages.models import Page


@modify_settings(
    INSTALLED_APPS={
        'append': 'glitter.tests.sample',
    },
)
@override_settings(
    PASSWORD_HASHERS=('django.contrib.auth.hashers.MD5PasswordHasher',),
    TEMPLATE_DIRS=(os.path.join(os.path.dirname(__file__), 'templates'),),
    ROOT_URLCONF='glitter.tests.urls',
)
class TestAdmin(TestCase):

    def setUp(self):
        User = get_user_model()
        # Page
        self.page = Page.objects.create(url='/test/', title='Test page')

        # Permissions
        self.edit_permissions = Permission.objects.get_by_natural_key(
            'edit_page', 'glitter_pages', 'page'
        )

        # Editor with editing permissions
        self.editor = User.objects.create_user('editor', 'editor@test.com', 'editor')
        self.editor.is_staff = True
        self.editor.user_permissions.add(self.edit_permissions)
        self.editor.save()
        self.editor_client = Client()
        self.editor_client.login(username='editor', password='editor')

        # Page version.
        self.page_version = Version.objects.create(
            content_type=ContentType.objects.get_for_model(Page), object_id=self.page.id,
            template_name='glitter/sample.html', owner=self.editor
        )

        self.html1_block = HTML.objects.create(content='<p>HTML Block</p>')
        self.html1_content_block = ContentBlock.objects.create(
            obj_version=self.page_version,
            column='main_content',
            position=1,
            content_type=ContentType.objects.get_for_model(self.html1_block),
            object_id=self.html1_block.id
        )
        self.html1_block.content_block = self.html1_content_block
        self.html1_block.save()

        # Information about model
        self.info = self.page._meta.app_label, self.page._meta.model_name

        self.html1_block_delete_url = reverse(
            'admin:%s_%s_block_delete' % self.info, args=(self.html1_content_block.id,)
        )



# -*- coding: utf-8 -*-

from django.apps import AppConfig


class GlittersConfig(AppConfig):
    name = 'glitter'
    label = 'glitter'

    def ready(self):
        super(GlittersConfig, self).ready()
        self.module.autodiscover()



# -*- coding: utf-8 -*-

import os

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase, Client
from django.test import override_settings, modify_settings

from glitter.exceptions import GlitterRedirectException
from glitter.pages.models import Page
from glitter.models import Version
from glitter.page import Glitter


class MockRequest(object):
    pass


class MockSuperUser(object):
    def has_perm(self, perm):
        return True

request = MockRequest()
request.user = MockSuperUser()


@modify_settings(
    INSTALLED_APPS={
        'append': 'glitter.tests.sample',
    },
)
@override_settings(ROOT_URLCONF='glitter.tests.urls')
@override_settings(
    PASSWORD_HASHERS=('django.contrib.auth.hashers.MD5PasswordHasher',),
    TEMPLATE_DIRS=(os.path.join(os.path.dirname(__file__), 'templates'),),
    ROOT_URLCONF='glitter.tests.urls',
)
class TestExceptionsBase(TestCase):
    def setUp(self):
        User = get_user_model()

        # Page
        self.page = Page.objects.create(url='/test/', title='Test page')

        # Permissions
        self.edit_permissions = Permission.objects.get_by_natural_key(
            'edit_page', 'glitter_pages', 'page'
        )
        # Editor with editing permissions
        self.editor = User.objects.create_user('editor', 'editor@test.com', 'editor')
        self.editor.is_staff = True
        self.editor.user_permissions.add(self.edit_permissions)
        self.editor.save()
        self.editor_client = Client()
        self.editor_client.login(username='editor', password='editor')

        # Page version.
        self.page_version = Version.objects.create(
            content_type=ContentType.objects.get_for_model(Page), object_id=self.page.id,
            template_name='glitter/sample.html', owner=self.editor
        )
        self.glitter = Glitter(self.page_version)


class TestGlitterColumnException(TestExceptionsBase):
    def setUp(self):
        super(TestGlitterColumnException, self).setUp()

    def test_page_redirect_exception(self):
        url = '/'
        exception = GlitterRedirectException(url)
        self.assertEqual(exception.__str__(), 'Redirect to %s required' % (url,))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import re

from django.db import models

from glitter.models import BaseBlock

from .validators import validate_url, YOUTUBE_URL_RE, VIMEO_URL_RE


class Video(BaseBlock):
    url = models.URLField(
        'URL',
        help_text='YouTube, Vimeo videos only',
        validators=[validate_url],
    )
    html = models.TextField(editable=False)

    class Meta:
        verbose_name = 'video'

    def get_embed_url(self):
        """ Get correct embed url for Youtube or Vimeo. """
        embed_url = None
        youtube_embed_url = 'https://www.youtube.com/embed/{}'
        vimeo_embed_url = 'https://player.vimeo.com/video/{}'

        # Get video ID from url.
        if re.match(YOUTUBE_URL_RE, self.url):
            embed_url = youtube_embed_url.format(re.match(YOUTUBE_URL_RE, self.url).group(2))
        if re.match(VIMEO_URL_RE, self.url):
            embed_url = vimeo_embed_url.format(re.match(VIMEO_URL_RE, self.url).group(3))
        return embed_url

    def save(self, force_insert=False, force_update=False, using=None, update_fields=None):
        """ Set html field with correct iframe. """
        if self.url:
            self.html = """<iframe src="{}" frameborder="0" allowfullscreen></iframe>""".format(
                self.get_embed_url()
            )
        return super(Video, self).save(force_insert, force_update, using, update_fields)



# -*- coding: utf-8 -*-


class Column(object):

    creation_counter = 0

    def __init__(self, verbose_name=None, width=0):
        self.verbose_name = verbose_name
        self.width = width

        self.creation_counter = Column.creation_counter
        Column.creation_counter += 1



# -*- coding: utf-8 -*-

from unittest import skipIf

from django.conf import settings
from django.contrib.admin.sites import AdminSite
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.test import TestCase, Client
from django.test import override_settings, modify_settings

from glitter.block_admin import BlockAdminSite, BlockModelAdmin
from glitter.blocks.html.models import HTML
from glitter.pages.admin import PageAdmin
from glitter.pages.models import Page
from glitter.models import ContentBlock, Version


SAMPLE_BLOCK_MISSING = 'glitter.tests.sampleblocks' not in settings.INSTALLED_APPS
if not SAMPLE_BLOCK_MISSING:
    from glitter.tests.sampleblocks.admin import SampleInlineAdmin


class MockRequest(object):
    pass


class MockSuperUser(object):
    def has_perm(self, perm):
        return True

request = MockRequest()
request.user = MockSuperUser()


@modify_settings(
    INSTALLED_APPS={
        'append': 'glitter.tests.sampleblocks',
    },
)
@override_settings(ROOT_URLCONF='glitter.tests.urls')
@override_settings(
    PASSWORD_HASHERS=('django.contrib.auth.hashers.MD5PasswordHasher',),
)
class TestBlockAdminSite(TestCase):
    def setUp(self):
        self.site = BlockAdminSite(name='block_admin')
        self.site.register(HTML)

        self.page_admin = PageAdmin(Page, AdminSite())

        User = get_user_model()
        # Superuser
        self.super_user = User.objects.create_superuser('test', 'test@test.com', 'test')
        self.super_user_client = Client()
        self.super_user_client.login(username='test', password='test')

        # Normal user
        self.normal_user = User.objects.create_user('name', 'test1@test.com', 'psw')
        self.normal_user_client = Client()
        self.normal_user_client.login(username='name', password='psw')

    def test_register_with_admin_class(self):
        self.site.unregister(HTML)
        self.site.register(HTML, BlockModelAdmin)

    def test_register_block(self):
        self.site.register_block(HTML, 'Common')

        # Run again whe `Common` is added in self.block_list
        self.site.register_block(HTML, 'Common')

    def test_get_urls(self):
        with self.settings(DEBUG=True):
            self.site.get_urls()
        self.site.get_urls()

    def test_admin_view(self):
        # Cacheble
        jsi18n_url = reverse('block_admin:jsi18n')
        response = self.super_user_client.get(jsi18n_url)
        self.assertEqual(response.status_code, 200)

        # Not cacheble
        jsi18n_url = reverse('block_admin:jsi18n')
        response = self.super_user_client.get(jsi18n_url, {'cacheable': False})
        self.assertEqual(response.status_code, 200)

        response = self.normal_user_client.get(jsi18n_url)
        self.assertEqual(response.status_code, 403)


@modify_settings(
    INSTALLED_APPS={
        'append': 'glitter.tests.sampleblocks',
    },
)
@override_settings(ROOT_URLCONF='glitter.tests.urls')
class TestBlockModelAdmin(TestCase):
    def setUp(self):

        User = get_user_model()
        # Superuser
        self.superuser = User.objects.create_superuser('test', 'test@test.com', 'test')
        self.superuser_client = Client()
        self.superuser_client.login(username='test', password='test')

        # Normal user
        self.normal_user = User.objects.create_user('name', 'test1@test.com', 'psw')
        self.normal_user.is_staff = True
        self.normal_user.save()
        self.normal_user_client = Client()
        self.normal_user_client.login(username='name', password='psw')

        # Page
        self.page = Page.objects.create(url='/test/', title='Test page')

        # Page version.
        self.page_version = Version.objects.create(
            content_type=ContentType.objects.get_for_model(Page), object_id=self.page.id,
            template_name='glitter/sample.html', owner=self.superuser
        )
        self.html_block = HTML.objects.create(content='<p>HTML Block</p>')
        self.content_block = ContentBlock.objects.create(
            obj_version=self.page_version,
            column='main_content',
            position=1,
            content_type=ContentType.objects.get_for_model(self.html_block),
            object_id=self.html_block.id
        )
        self.html_block.content_block = self.content_block
        self.html_block.save()

        self.site = AdminSite()
        self.model_admin = BlockModelAdmin(HTML, self.site)

        # Information about model
        self.info = self.model_admin.model._meta.app_label, self.model_admin.model._meta.model_name

        self.continue_view_url = reverse(
            'block_admin:%s_%s_continue' % self.info, args=(self.html_block.id,)
        )

        self.change_view_url = reverse(
            'block_admin:%s_%s_change' % self.info, args=(self.html_block.id,)
        )

    def test_change_form_template(self):
        self.model_admin.change_form_template

    def test_admin_view(self):
        jsi18n_url = reverse('block_admin:jsi18n')
        response = self.superuser_client.get(jsi18n_url)
        self.assertEqual(response.status_code, 200)

    def test_continue_view(self):
        response = self.superuser_client.get(self.continue_view_url)
        self.assertEqual(response.status_code, 200)

        continue_view_url_does_not_exit = reverse(
            'block_admin:%s_%s_continue' % self.info, args=('4')
        )
        response = self.superuser_client.get(continue_view_url_does_not_exit)
        self.assertEqual(response.status_code, 404)

    def test_continue_view_permissions(self):
        # Permission denied as user doesn't have permissions
        response = self.normal_user_client.get(self.continue_view_url)
        self.assertEqual(response.status_code, 403)

    def test_change_view(self):
        response = self.superuser_client.get(self.change_view_url)
        self.assertEqual(response.status_code, 200)

    def test_change_view_permissions(self):
        # Permission denied as user doesn't have permissions
        response = self.normal_user_client.get(self.change_view_url)
        self.assertEqual(response.status_code, 403)

        change_view_url_does_not_exit = reverse(
            'block_admin:%s_%s_change' % self.info, args=('4')
        )
        response = self.superuser_client.get(change_view_url_does_not_exit)
        self.assertEqual(response.status_code, 404)

    def test_change_view_page_version(self):
        self.page_version.version_number = 1
        self.page_version.save()
        response = self.superuser_client.get(self.change_view_url)
        self.assertEqual(response.status_code, 403)

    def test_response_change(self):
        """Test response change with and without _continue post."""
        opts = self.html_block._meta.app_label, self.html_block._meta.model_name
        cat = reverse('block_admin:%s_%s_change' % opts, args=(self.html_block.id,))
        response = self.superuser_client.post(cat, {
            'content': '<h1>Test</h1>',
            '_continue': True,
        })
        self.assertEqual(response.status_code, 302)
        response = self.superuser_client.post(cat, {
            'content': '<h1>Test</h1>',
        })
        self.assertEqual(response.status_code, 200)


@skipIf(SAMPLE_BLOCK_MISSING, 'glitter.tests.sampleblocks is not installed')
class TestInlineBlockModelAdmin(TestCase):
    def setUp(self):
        # Superuser
        User = get_user_model()
        self.superuser = User.objects.create_superuser('test', 'test@test.com', 'test')
        self.superuser_client = Client()
        self.superuser_client.login(username='test', password='test')

        self.model_admin = SampleInlineAdmin(Page, AdminSite())

    def test_has_add_permission(self):
        request = MockRequest()
        request.user = self.superuser
        self.assertTrue(self.model_admin.has_add_permission(request))

    def test_has_change_permission(self):
        request = MockRequest()
        request.user = self.superuser
        self.assertTrue(self.model_admin.has_change_permission(request))

    def test_has_delete_permission(self):
        request = MockRequest()
        request.user = self.superuser
        self.assertTrue(self.model_admin.has_delete_permission(request))



# -*- coding: utf-8 -*-

from itertools import groupby

from django.db import models
from django.forms.models import ModelChoiceField, ModelChoiceIterator


class GroupedModelChoiceField(ModelChoiceField):
    def __init__(self, queryset, group_by_field='category', group_label=None, *args, **kwargs):
        """
        group_by_field is the name of a field on the model
        group_label is a function to return a label for each choice group
        """
        super(GroupedModelChoiceField, self).__init__(queryset, *args, **kwargs)
        self.group_by_field = group_by_field
        if group_label is None:
            self.group_label = lambda group: group
        else:
            self.group_label = group_label

        # Need correct ordering, and category titles
        self.queryset = self.queryset.select_related().order_by('category', 'title')

    def _get_choices(self):
        """
        Exactly as per ModelChoiceField except returns new iterator class
        """
        if hasattr(self, '_choices'):
            return self._choices
        return GroupedModelChoiceIterator(self)
    choices = property(_get_choices, ModelChoiceField._set_choices)


class GroupedModelChoiceIterator(ModelChoiceIterator):
    def __iter__(self):
        if self.field.empty_label is not None:
            yield ('', self.field.empty_label)

        if self.field.cache_choices:
            if self.field.choice_cache is None:
                self.field.choice_cache = [
                    (self.field.group_label(group), [self.choice(ch) for ch in choices])
                    for group, choices in groupby(
                        self.queryset.all(), key=lambda row: getattr(
                            row, self.field.group_by_field
                        )
                    )
                ]

            for choice in self.field.choice_cache:
                yield choice
        else:
            for group, choices in groupby(
                    self.queryset.all(),
                    key=lambda row: getattr(row, self.field.group_by_field)
            ):
                yield (self.field.group_label(group), [self.choice(ch) for ch in choices])


class AssetForeignKey(models.ForeignKey):
    def formfield(self, **kwargs):
        kwargs.setdefault('form_class', GroupedModelChoiceField)
        return super(AssetForeignKey, self).formfield(**kwargs)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.encoding import python_2_unicode_compatible

from glitter.models import BaseBlock


class DefinitionList(BaseBlock):
    class Meta:
        verbose_name = 'Definition list'


@python_2_unicode_compatible
class DefinitionListInline(models.Model):
    definition_list = models.ForeignKey(DefinitionList)
    key = models.CharField(max_length=128)
    value = models.TextField()

    def __str__(self):
        return ''



# -*- coding: utf-8 -*-

from collections import namedtuple

from django.apps import apps
from django.db.models.base import ModelBase
from django.utils import six

from .layouts import PageLayoutBase


Template = namedtuple('Template', ['layout', 'model'])


def register(class_or_iterable, model_class):
    if isinstance(class_or_iterable, PageLayoutBase):
        class_or_iterable = [class_or_iterable]

    for layout in class_or_iterable:
        template_name = layout._meta.template

        # Find the model class given a string if needed
        if not isinstance(model_class, ModelBase):
            model_class = apps.get_model(model_class)

        templates[template_name] = Template(layout=layout, model=model_class)


def get_templates(model):
    """ Return a list of templates usable by a model. """
    for template_name, template in six.iteritems(templates):
        if issubclass(template.model, model):
            yield (template_name, template.layout._meta.verbose_name)


def get_layout(template_name):
    """ Return a registered layout from a template name. """
    return templates[template_name].layout


def attach(*layouts, **kwargs):
    """
    Registers the given layout(s) classes
    admin site:

    @pages.register(Page)
    class Default(PageLayout):
        pass
    """

    def _model_admin_wrapper(layout_class):
        register(layout_class, layouts[0])
        return layout_class
    return _model_admin_wrapper

templates = {}



# -*- coding: utf-8 -*-

from django.contrib.auth import get_user_model
from django.test import TestCase, Client
from django.test import override_settings

from glitter.pages.models import Page
from glitter.models import Version


@override_settings(ROOT_URLCONF='glitter.tests.urls')
@override_settings(PASSWORD_HASHERS=('django.contrib.auth.hashers.MD5PasswordHasher',))
class TestListeners(TestCase):
    def setUp(self):
        # Page
        self.page = Page.objects.create(url='/test/', title='Test page')

        User = get_user_model()
        # Superuser
        self.superuser = User.objects.create_superuser('test', 'test@test.com', 'test')
        self.superuser_client = Client()
        self.superuser_client.login(username='test', password='test')

        # Page version.
        self.page_version = Version.objects.create(
            page=self.page, template_name='glitter/sample.html', owner=self.superuser
        )



# -*- coding: utf-8 -*-

from django.apps import AppConfig


class GlitterBasicAssetsConfig(AppConfig):
    name = 'glitter.assets'
    label = 'glitter_assets'
    verbose_name = 'Assets'

    def ready(self):
        super(GlitterBasicAssetsConfig, self).ready()
        from . import listeners  # noqa



# -*- coding: utf-8 -*-

from django import forms
from django.conf import settings
from django.utils.encoding import force_text
from django.utils.text import capfirst

from . import block_admin
from .models import ContentBlock, Version
from .templates import get_layout, get_templates


def get_addblock_form(page_version):
    template_obj = get_layout(template_name=page_version.template_name)

    block_choices = []

    # Group all block by category
    for category in sorted(block_admin.site.block_list):
        category_blocks = block_admin.site.block_list[category]
        category_choices = (('%s.%s' % (x._meta.app_label, x._meta.object_name),
                             capfirst(force_text(x._meta.verbose_name))) for x in category_blocks)
        category_choices = sorted(category_choices, key=lambda x: x[1])
        block_choices.append((category, category_choices))

    class AddBlockForm(forms.ModelForm):
        class Meta:
            model = ContentBlock
            fields = ('column',)

        block_type = forms.ChoiceField(choices=block_choices)
        column = forms.ChoiceField(
            choices=((x, x) for x in template_obj._meta.columns)
        )
        top = forms.BooleanField(required=False)

    return AddBlockForm


class BaseNewPageTemplateForm(forms.ModelForm):
    class Meta:
        model = Version
        fields = ('template_name',)

    template_name = forms.ChoiceField(choices=())


def get_newpagetemplateform(model):
    template_choices = sorted(get_templates(model=model), key=lambda x: x[1])
    initial_template = getattr(settings, 'GLITTER_DEFAULT_TEMPLATE', None)

    class NewPageTemplateForm(BaseNewPageTemplateForm):
        template_name = forms.ChoiceField(choices=template_choices, initial=initial_template)

    return NewPageTemplateForm


class MoveBlockForm(forms.Form):
    MOVE_TOP = 1
    MOVE_UP = 2
    MOVE_DOWN = 3
    MOVE_BOTTOM = 4

    move = forms.IntegerField(min_value=MOVE_TOP, max_value=MOVE_BOTTOM)


def get_movecolumn_form(columns):
    move_choices = ((x, x) for x in columns)

    class MoveColumnForm(forms.Form):
        move = forms.ChoiceField(choices=move_choices)

    return MoveColumnForm



from glitter import block_admin

from .models import RelatedPage, RelatedPagesBlock


class RelatedPageInline(block_admin.StackedInline):
    model = RelatedPage
    extra = 1


class RelatedPagesBlockAdmin(block_admin.BlockModelAdmin):
    inlines = [RelatedPageInline]


block_admin.site.register(RelatedPagesBlock, RelatedPagesBlockAdmin)
block_admin.site.register_block(RelatedPagesBlock, 'App Blocks')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('glitter', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='DefinitionList',
            fields=[
                ('id', models.AutoField(verbose_name='ID', auto_created=True, primary_key=True, serialize=False)),
                ('content_block', models.ForeignKey(editable=False, to='glitter.ContentBlock', null=True)),
            ],
            options={
                'verbose_name': 'Definition list',
            },
        ),
        migrations.CreateModel(
            name='DefinitionListInline',
            fields=[
                ('id', models.AutoField(verbose_name='ID', auto_created=True, primary_key=True, serialize=False)),
                ('key', models.CharField(max_length=128)),
                ('value', models.TextField()),
                ('definition_list', models.ForeignKey(to='glitter_definition_list.DefinitionList')),
            ],
        ),
    ]



# -*- coding: utf-8 -*-

from django.contrib.auth import get_user_model
from django.core.exceptions import ValidationError
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase, Client, override_settings

from glitter.models import Version, ContentBlock
from glitter.pages.models import Page
from glitter.pages.validators import validate_page_url
from glitter.blocks.html.models import HTML


@override_settings(PASSWORD_HASHERS=('django.contrib.auth.hashers.MD5PasswordHasher',))
class TestModelsPage(TestCase):
    def setUp(self):
        # Page
        self.page = Page.objects.create(url='/test/', title='Test page')

        User = get_user_model()
        # Editor with not editing permissions
        self.editor_no_permissions = User.objects.create_user(
            'editor_no_perm', 'editor_no_perm@test.com', 'editor_no_perm'
        )
        self.editor_no_permissions.is_staff = True
        self.editor_no_permissions.save()
        self.editor_no_permissions_client = Client()
        self.editor_no_permissions_client.login(
            username='editor_no_perm', password='editor_no_perm'
        )

        page_content_type = ContentType.objects.get_for_model(Page)

        # Page version.
        self.page_version = Version.objects.create(
            content_type=page_content_type,
            object_id=self.page.id,
            template_name='glitter/sample.html',
            owner=self.editor_no_permissions
        )
        # Page version.
        self.page_version_1 = Version.objects.create(
            content_type=page_content_type,
            object_id=self.page.id,
            template_name='glitter/sample.html',
            owner=self.editor_no_permissions
        )

        self.html1_block = HTML.objects.create(content='<p>HTML Block</p>')
        self.html1_content_block = ContentBlock.objects.create(
            obj_version=self.page_version,
            column='Main Content',
            position=1,
            content_type=ContentType.objects.get_for_model(self.html1_block),
            object_id=self.html1_block.id
        )
        self.html1_block.content_block = self.html1_content_block
        self.html1_block.save()

        self.html2_block = HTML.objects.create(content='<p>HTML Block</p>')
        self.html2_content_block = ContentBlock.objects.create(
            obj_version=self.page_version,
            column='Main Content',
            position=2,
            content_type=ContentType.objects.get_for_model(self.html2_block),
            object_id=self.html2_block.id
        )
        self.html2_block.content_block = self.html2_content_block
        self.html2_block.save()

        self.html3_block = HTML.objects.create(content='<p>HTML Block</p>')
        self.html3_content_block = ContentBlock.objects.create(
            obj_version=self.page_version,
            column='Main Content',
            position=3,
            content_type=ContentType.objects.get_for_model(self.html3_block),
            object_id=self.html3_block.id
        )
        self.html3_block.content_block = self.html3_content_block
        self.html3_block.save()

    def test_validate_page_url(self):
        self.assertRaises(ValidationError, validate_page_url, value='admin')
        self.assertRaises(ValidationError, validate_page_url, value='/admin')
        self.assertRaises(ValidationError, validate_page_url, value='//')

    def test_manager_published(self):
        self.page_version.generate_version()
        self.page_version.save()

        self.page.published = True
        self.page.current_version = self.page_version
        self.page.save()

        self.assertEqual(1, Page.objects.published().count())

    def test_page_version_str(self):
        self.page_version.__str__()

        # Test __str__ by adding version number.
        self.page_version.version_number = 1
        self.page_version.save()
        self.page_version.__str__()

    def test_generate_version(self):
        self.page_version.generate_version()

        # Test removing version number
        self.page_version.version_number = 1
        self.page_version.save()
        self.page_version.generate_version()

    def test_is_published(self):
        self.page_version.is_published

        # Test removing version number
        self.page_version.version_number = 1
        self.page_version.save()
        self.page.current_version = self.page_version

        self.page_version.is_published

    def test_content_block_last_position(self):
        """Test if position is added if last_position is None."""

        ContentBlock.objects.update(position=None)

        self.html1_content_block.position = None
        self.html1_content_block.save()
        self.assertEqual(self.html1_content_block.position, 1)

        self.html2_content_block.position = None
        self.html2_content_block.save()
        self.assertEqual(self.html1_content_block.position, 1)



# -*- coding: utf-8 -*-

import os

from django.contrib.auth import get_user_model
from django.contrib.admin.sites import AdminSite
from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.http import HttpRequest
from django.test import TestCase, Client
from django.test import override_settings, modify_settings
from django.conf import settings

from glitter.forms import MoveBlockForm
from glitter.blocks.html.models import HTML
from glitter.models import Version, ContentBlock
from glitter.pages.admin import PageAdmin, page_admin_fields
from glitter.pages.models import Page
from glitter.tests.sample.models import Book


SAMPLE_BLOCK_MISSING = 'glitter.tests.sampleblocks' not in settings.INSTALLED_APPS


@modify_settings(
    INSTALLED_APPS={
        'append': 'glitter.tests.sample',
    },
)
@override_settings(
    PASSWORD_HASHERS=('django.contrib.auth.hashers.MD5PasswordHasher',),
    TEMPLATE_DIRS=(os.path.join(os.path.dirname(__file__), 'templates'),),
    ROOT_URLCONF='glitter.tests.urls',
    GLITTER_SHOW_LOGIN_REQUIRED=True,
)
class TestAdmin(TestCase):

    def setUp(self):

        # Permissions
        self.edit_permissions = Permission.objects.get_by_natural_key(
            'edit_page', 'glitter_pages', 'page'
        )

        # Page
        self.page = Page.objects.create(url='/test/', title='Test page')

        # Information about model
        self.info = self.page._meta.app_label, self.page._meta.model_name

        User = get_user_model()
        # Superuser
        self.super_user = User.objects.create_superuser('test', 'test@test.com', 'test')
        self.super_user_client = Client()
        self.super_user_client.login(username='test', password='test')

        # Editor with editing permissions
        self.editor = User.objects.create_user('editor', 'editor@test.com', 'editor')
        self.editor.is_staff = True
        self.editor.user_permissions.add(self.edit_permissions)
        self.editor.save()
        self.editor_client = Client()
        self.editor_client.login(username='editor', password='editor')

        # Editor with not editing permissions
        self.editor_no_permissions = User.objects.create_user(
            'editor_no_perm', 'editor_no_perm@test.com', 'editor_no_perm'
        )
        self.editor_no_permissions.is_staff = True
        self.editor_no_permissions.save()
        self.editor_no_permissions_client = Client()
        self.editor_no_permissions_client.login(
            username='editor_no_perm', password='editor_no_perm'
        )

        # Page version.
        self.page_version = Version.objects.create(
            content_type=ContentType.objects.get_for_model(Page), object_id=self.page.id,
            template_name='glitter/sample.html', owner=self.editor
        )

        self.page_admin = PageAdmin(Page, AdminSite())

        self.change_obj_url = reverse('admin:glitter_pages_page_change', args=(self.page.id,))
        self.add_obj_url = reverse('admin:glitter_pages_page_add')
        self.page_redirect_url = reverse('admin:glitter_pages_page_redirect', args=(self.page.id,))

    def test_data_for_change_and_add_response(self):
        response_data = {'url': '/testing2/', 'title': 'Testing2'}
        response_save_and_edit_data = {
            'url': '/testing223/', 'title': 'Testing2', '_saveandedit': True
        }
        response_save_and_continue_edit = {
            'url': '/testing223/', 'title': 'Testing2', '_continue': True
        }
        return response_data, response_save_and_edit_data, response_save_and_continue_edit

    def test_admin_url(self):
        self.page_admin.view_url(self.page)

    def test_unpublish_count(self):
        self.page_admin.admin_unpublished_count(self.page)

    def test_response_change(self):
        response_data, response_save_and_edit_data, response_save_and_continue_edit = (
            self.test_data_for_change_and_add_response()
        )
        self.super_user_client.post(self.change_obj_url, response_save_and_edit_data)
        self.super_user_client.post(self.change_obj_url, response_data)
        self.super_user_client.post(self.change_obj_url, response_save_and_continue_edit)
        self.super_user_client.get(self.page_redirect_url)

    def test_response_add(self):
        response_data, response_save_and_edit_data, response_save_and_continue_edit = (
            self.test_data_for_change_and_add_response()
        )
        response_data['_saveasnew'] = 'Testing'
        self.super_user_client.post(self.add_obj_url, response_data)
        self.super_user_client.post(self.add_obj_url, response_save_and_edit_data)
        self.super_user_client.post(self.add_obj_url, response_save_and_continue_edit)
        self.super_user_client.get(self.page_redirect_url)

    def test_show_login(self):
        self.assertEqual(
            page_admin_fields(),
            ['url', 'title', 'parent', 'login_required', 'show_in_navigation']
        )


@modify_settings(
    INSTALLED_APPS={
        'append': 'glitter.tests.sample',
    },
)
class TestPermissions(TestCase):
    def setUp(self):
        # Permissions for objects we're testing
        self.edit_page = Permission.objects.get_by_natural_key(
            'edit_page', 'glitter_pages', 'page'
        )
        self.publish_page = Permission.objects.get_by_natural_key(
            'publish_page', 'glitter_pages', 'page'
        )
        self.edit_book = Permission.objects.get_by_natural_key(
            'edit_book', 'sample', 'book'
        )
        self.publish_book = Permission.objects.get_by_natural_key(
            'publish_book', 'sample', 'book'
        )

        # Superuser
        User = get_user_model()
        self.superuser = User.objects.create_superuser(
            username='superuser', email='', password=None
        )

        # Editor with editing permissions
        self.editor = User.objects.create_user(username='editor', email='', password=None)
        self.editor.is_staff = True
        self.editor.save()
        self.editor.user_permissions.add(self.edit_page, self.edit_book)

        # Publisher with edit and publish permissions
        self.publisher = User.objects.create_user(username='publisher', email='', password=None)
        self.publisher.is_staff = True
        self.publisher.save()
        self.publisher.user_permissions.add(
            self.edit_page, self.publish_page, self.edit_book, self.publish_book
        )

        # Staff with no editing permissions
        self.staff = User.objects.create_user(username='staff', email='', password=None)
        self.staff.is_staff = True
        self.staff.save()

        # Page with an unsaved page version
        self.page = Page.objects.create(url='/test/', title='Test page')
        self.page_version = Version.objects.create(
            content_type=ContentType.objects.get_for_model(Page),
            object_id=self.page.id,
            template_name='glitter/sample.html',
            owner=self.editor,
        )
        self.page_admin = PageAdmin(Page, AdminSite())

        # Sample model
        self.book = Book.objects.create(title='Hello')
        self.book_admin = PageAdmin(Book, AdminSite())

    def test_edit_permission(self):
        # Only people with glitter_pages.edit_page have edit permission
        request = HttpRequest()

        request.user = self.editor
        self.assertTrue(self.page_admin.has_edit_permission(request=request))

        request.user = self.staff
        self.assertFalse(self.page_admin.has_edit_permission(request=request))

    def test_edit_version(self):
        # Only the creator of an unsaved version can edit it
        request = HttpRequest()

        request.user = self.superuser
        self.assertFalse(self.page_admin.has_edit_permission(
            request=request, version=self.page_version
        ))

        request.user = self.editor
        self.assertTrue(self.page_admin.has_edit_permission(
            request=request, version=self.page_version
        ))

    def test_publish_permission(self):
        # Only people with glitter_pages.publish_page have publish permission
        request = HttpRequest()

        request.user = self.publisher
        self.assertTrue(self.page_admin.has_publish_permission(request=request))

        request.user = self.staff
        self.assertFalse(self.page_admin.has_publish_permission(request=request))

    def test_book_model(self):
        # Test that permissions work with different types of models
        request = HttpRequest()

        request.user = self.editor
        self.assertTrue(self.book_admin.has_edit_permission(request=request))

        request.user = self.publisher
        self.assertTrue(self.book_admin.has_publish_permission(request=request))

        request.user = self.staff
        self.assertFalse(self.book_admin.has_edit_permission(request=request))
        self.assertFalse(self.book_admin.has_publish_permission(request=request))


class BaseViewsCase(TestAdmin):

    def setUp(self):
        super(BaseViewsCase, self).setUp()

        self.html1_block = HTML.objects.create(content='<p>HTML Block</p>')
        self.html1_content_block = ContentBlock.objects.create(
            obj_version=self.page_version,
            column='main_content',
            position=1,
            content_type=ContentType.objects.get_for_model(self.html1_block),
            object_id=self.html1_block.id
        )
        self.html1_block.content_block = self.html1_content_block
        self.html1_block.save()

        self.html2_block = HTML.objects.create(content='<p>HTML Block</p>')
        self.html2_content_block = ContentBlock.objects.create(
            obj_version=self.page_version,
            column='main_content',
            position=3,
            content_type=ContentType.objects.get_for_model(self.html2_block),
            object_id=self.html2_block.id
        )
        self.html2_block.content_block = self.html2_content_block
        self.html2_block.save()

        self.html3_block = HTML.objects.create(content='<p>HTML Block</p>')
        self.html3_content_block = ContentBlock.objects.create(
            obj_version=self.page_version,
            column='side',
            position=4,
            content_type=ContentType.objects.get_for_model(self.html3_block),
            object_id=self.html3_block.id
        )
        self.html3_block.content_block = self.html3_content_block
        self.html3_block.save()

    def change_page_version(self):
        self.page_version.version_number = 1
        self.page_version.save()


class TestPageChangeTemplateView(BaseViewsCase):
    def setUp(self):
        super(TestPageChangeTemplateView, self).setUp()
        self.change_template_url = reverse(
            'admin:%s_%s_changetemplate' % self.info, args=(self.page_version.id,)
        )

    def test_permissions(self):
        # Permission denied as user doesn't have permissions
        response = self.editor_no_permissions_client.get(self.change_template_url)
        self.assertEqual(response.status_code, 403)

        # Editor with permissions
        response = self.editor_client.get(self.change_template_url)
        self.assertEqual(response.status_code, 302)

    def test_post(self):
        # Check POST
        response = self.editor_client.post(self.change_template_url, {
            'template_name': 'glitter/sample2.html',
        })
        self.assertEqual(response.status_code, 302)

        # Check template gets changed
        page_version = Version.objects.get(id=self.page_version.id)
        self.assertEqual(page_version.template_name, 'glitter/sample2.html')

        # These blocks get moved to a new column
        html1_content_block = ContentBlock.objects.get(id=self.html1_content_block.id)
        html2_content_block = ContentBlock.objects.get(id=self.html2_content_block.id)

        self.assertEqual(html1_content_block.column, 'content')
        self.assertEqual(html2_content_block.column, 'content')
        self.assertEqual(html1_content_block.position, 1)
        self.assertEqual(html2_content_block.position, 2)

        # Index error if ContentBlock doesn't exist
        self.editor_client.post(self.change_template_url, {
            'template_name': 'glitter/sample.html',
        })

    def test_page_owner(self):
        """ Check page owner. """
        self.page_version.owner = self.super_user
        self.page_version.save()
        response = self.editor_client.get(self.change_template_url)
        self.assertEqual(response.status_code, 403)

    def test_page_version(self):
        """ Check page version. """
        self.change_page_version()
        self.editor.user_permissions.add(self.edit_permissions)
        response = self.editor_client.get(self.change_template_url)
        self.assertEqual(response.status_code, 403)


class TestPageBlockAddView(BaseViewsCase):
    def setUp(self):
        super(TestPageBlockAddView, self).setUp()
        self.page_block_add_view_url = reverse(
            'admin:%s_%s_block_add' % self.info, args=(self.page_version.id,)
        )

    def test_permissions(self):
        # Permission denied as user doesn't have permissions
        response = self.editor_no_permissions_client.get(self.page_block_add_view_url)
        self.assertEqual(response.status_code, 403)

        # Editor with permissions
        response = self.editor_client.get(self.page_block_add_view_url)
        self.assertEqual(response.status_code, 200)

    def test_post(self):
        response = self.editor_client.post(self.page_block_add_view_url, {
            'column': 'main_content',
            'block_type': u'glitter_html_block.HTML',
            'top': 'true',
        })
        self.assertEqual(response.status_code, 200)

        response = self.editor_client.post(self.page_block_add_view_url, {
            'column': 'main_content',
            'block_type': u'glitter_html_block.HTML',
        })
        self.assertEqual(response.status_code, 200)

    def test_page_version(self):
        """ Check page version. """
        self.change_page_version()
        self.editor.user_permissions.add(self.edit_permissions)
        response = self.editor_client.get(self.page_block_add_view_url)
        self.assertEqual(response.status_code, 403)


class TestPageBlockDeleteView(BaseViewsCase):

    def setUp(self):
        super(TestPageBlockDeleteView, self).setUp()
        self.page_block_delete_view_url = reverse(
            'admin:%s_%s_block_delete' % self.info, args=(self.html1_content_block.id,)
        )

    def test_permissions(self):
        """ Test permissions. """
        response = self.editor_no_permissions_client.get(self.page_block_delete_view_url)
        self.assertEqual(response.status_code, 403)

        # Editor with permissions
        response = self.editor_client.get(self.page_block_delete_view_url)
        self.assertEqual(response.status_code, 200)

    def test_post(self):

        response = self.editor_client.post(self.page_block_delete_view_url, {
            'column': 'main_content',
            'block_type': u'glitter_html_block.HTML',
        })
        self.assertEqual(response.status_code, 200)

    def test_page_version(self):
        """ Check page version. """
        self.change_page_version()
        self.editor.user_permissions.add(self.edit_permissions)
        response = self.editor_client.get(self.page_block_delete_view_url)
        self.assertEqual(response.status_code, 403)


class TestPageBlockMoveView(BaseViewsCase):
    def setUp(self):
        super(TestPageBlockMoveView, self).setUp()
        self.html1_block_move_view_url = reverse(
            'admin:%s_%s_block_move' % self.info, args=(self.html1_content_block.id,)
        )
        self.html2_block_move_view_url = reverse(
            'admin:%s_%s_block_move' % self.info, args=(self.html2_content_block.id,)
        )
        self.html3_block_move_view_url = reverse(
            'admin:%s_%s_block_move' % self.info, args=(self.html3_content_block.id,)
        )

    def test_permissions(self):
        """ Test permissions. """
        response = self.editor_no_permissions_client.get(self.html1_block_move_view_url)
        self.assertEqual(response.status_code, 403)

        # Editor with permissions
        response = self.editor_client.get(self.html1_block_move_view_url)
        self.assertEqual(response.status_code, 200)

    def test_html1_move_block_bottom(self):
        """ Move block top. """
        response = self.editor_client.post(self.html1_block_move_view_url, {
            'move': MoveBlockForm.MOVE_BOTTOM,
        })
        self.assertEqual(response.status_code, 200)

    def test_html2_move_block_bottom(self):
        """ Move block up. """
        response = self.editor_client.post(self.html2_block_move_view_url, {
            'move': MoveBlockForm.MOVE_BOTTOM,
        })
        self.assertEqual(response.status_code, 200)

    def test_html3_move_block_bottom(self):
        """ Move block down. """
        response = self.editor_client.post(self.html3_block_move_view_url, {
            'move': MoveBlockForm.MOVE_BOTTOM
        })
        self.assertEqual(response.status_code, 200)

    def test_move_html1_block_top(self):
        # Move block bottom
        response = self.editor_client.post(self.html1_block_move_view_url, {
            'move': MoveBlockForm.MOVE_TOP
        })
        self.assertEqual(response.status_code, 200)

    def test_move_html2_block_top(self):
        # Move block bottom
        response = self.editor_client.post(self.html2_block_move_view_url, {
            'move': MoveBlockForm.MOVE_TOP
        })
        self.assertEqual(response.status_code, 200)

    def test_move_html1_block_up(self):
        # Move block bottom
        response = self.editor_client.post(self.html1_block_move_view_url, {
            'move': MoveBlockForm.MOVE_UP
        })
        self.assertEqual(response.status_code, 200)

    def test_move_html1_block_down(self):
        # Move block bottom
        response = self.editor_client.post(self.html1_block_move_view_url, {
            'move': MoveBlockForm.MOVE_DOWN
        })
        self.assertEqual(response.status_code, 200)

    def test_page_version(self):
        """ Check page version. """
        self.change_page_version()
        self.editor.user_permissions.add(self.edit_permissions)
        response = self.editor_client.get(self.html1_block_move_view_url)
        self.assertEqual(response.status_code, 403)


class TestPageBlockColumnView(BaseViewsCase):

    def setUp(self):
        super(TestPageBlockColumnView, self).setUp()
        self.html1_block_column_url = reverse(
            'admin:%s_%s_block_column' % self.info, args=(self.html1_content_block.id,)
        )

    def test_permissions(self):
        """ Test permissions. """
        response = self.editor_no_permissions_client.get(self.html1_block_column_url)
        self.assertEqual(response.status_code, 403)

        # Editor with permissions
        response = self.editor_client.get(self.html1_block_column_url)
        self.assertEqual(response.status_code, 200)

    def test_move_block_side(self):
        """ Move block side. """
        response = self.editor_client.post(self.html1_block_column_url, {
            'move': 'side',
        })
        self.assertEqual(response.status_code, 200)

    def test_get_last_block(self):
        self.html3_block.delete()
        self.html3_content_block.delete()

        response = self.editor_client.post(self.html1_block_column_url, {
            'move': 'side',
        })
        self.assertEqual(response.status_code, 200)

    def test_page_version(self):
        """ Check page version. """
        self.change_page_version()
        self.editor.user_permissions.add(self.edit_permissions)
        response = self.editor_client.get(self.html1_block_column_url)
        self.assertEqual(response.status_code, 403)



# -*- coding: utf-8 -*-

from glitter import block_admin

from .models import ContactFormBlock


block_admin.site.register(ContactFormBlock)
block_admin.site.register_block(ContactFormBlock, 'Forms')



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('glitter_pages', '0001_initial'),
        ('glitter', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ContactFormBlock',
            fields=[
                ('id', models.AutoField(primary_key=True, verbose_name='ID', serialize=False, auto_created=True)),
                ('recipient', models.EmailField(max_length=254)),
                ('content_block', models.ForeignKey(editable=False, null=True, to='glitter.ContentBlock')),
                ('success_page', mptt.fields.TreeForeignKey(null=True, to='glitter_pages.Page', on_delete=django.db.models.deletion.SET_NULL)),
            ],
            options={
                'verbose_name': 'contact form',
            },
        ),
    ]



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from functools import update_wrapper

from django.conf import settings
from django.conf.urls import url
from django.contrib import admin
from django.contrib.admin.options import csrf_protect_m
from django.core.urlresolvers import reverse

from django_mptt_admin.admin import DjangoMpttAdmin
from mptt.admin import MPTTModelAdmin

from glitter.admin import GlitterAdminMixin
from .models import Page


def page_admin_fields():
    fields = ['url', 'title', 'parent', 'login_required', 'show_in_navigation']

    # Don't show login_required unless needed
    if not getattr(settings, 'GLITTER_SHOW_LOGIN_REQUIRED', False):
        fields.remove('login_required')

    return fields


@admin.register(Page)
class PageAdmin(GlitterAdminMixin, DjangoMpttAdmin, MPTTModelAdmin):
    list_display = (
        'title', 'url', 'view_url', 'is_published', 'in_nav', 'admin_unpublished_count',
    )
    fields = page_admin_fields()
    mptt_level_indent = 25
    glitter_render = True
    change_list_template = 'admin/pages/page/change_list.html'

    def view_url(self, obj):
        info = self.model._meta.app_label, self.model._meta.model_name
        redirect_url = reverse('admin:%s_%s_redirect' % info, kwargs={'object_id': obj.id})
        return '<a href="%s">View page</a>' % (redirect_url)
    view_url.short_description = 'View page'
    view_url.allow_tags = True

    def in_nav(self, obj):
        return obj.show_in_navigation
    in_nav.boolean = True

    def admin_unpublished_count(self, obj):
        return obj.unpublished_count or ''
    admin_unpublished_count.short_description = 'Unpublished pages'
    admin_unpublished_count.allow_tags = True

    @csrf_protect_m
    def changelist_view(self, request, extra_context=None):
        template_response = super(PageAdmin, self).changelist_view(request, extra_context)
        template_response.template_name = 'admin/pages/page/change_list_tree.html'
        return template_response

    def get_urls(self):
        def wrap(view):
            def wrapper(*args, **kwargs):
                return self.admin_site.admin_view(view)(*args, **kwargs)
            return update_wrapper(wrapper, view)
        urlpatterns = super(PageAdmin, self).get_urls()

        info = self.model._meta.app_label, self.model._meta.model_name

        urlpatterns = [
            url(r'^$', wrap(self.grid_view)),
            url(r'^tree/$', wrap(self.changelist_view), name='%s_%s_tree' % info),
        ] + urlpatterns
        return urlpatterns
